from networkx.utils.backends import _dispatchable

__all__ = ["hnm_harary_graph", "hkn_harary_graph"]

@_dispatchable
def hnm_harary_graph(n, m, create_using=None): ...
@_dispatchable
def hkn_harary_graph(k, n, create_using=None): ...



import dataclasses
from _typeshed import Incomplete, Unused
from collections import defaultdict
from collections.abc import Generator
from dataclasses import dataclass
from logging import Logger
from typing import Final, overload
from typing_extensions import Self, deprecated

from ._fonttools_shims import _TTFont
from .drawing import DeviceGray, DeviceRGB, Number
from .enums import Align, TextEmphasis
from .syntax import PDFObject

LOGGER: Logger

# Only defined if harfbuzz is installed.
class HarfBuzzFont(Incomplete):  # derives from uharfbuzz.Font
    def __deepcopy__(self, _memo: object) -> Self: ...

@dataclass
class FontFace:
    __slots__ = ("family", "emphasis", "size_pt", "color", "fill_color")
    family: str | None
    emphasis: TextEmphasis | None
    size_pt: int | None
    color: DeviceGray | DeviceRGB | None
    fill_color: DeviceGray | DeviceRGB | None

    def __init__(
        self,
        family: str | None = None,
        emphasis=None,
        size_pt: int | None = None,
        color: int | tuple[Number, Number, Number] | DeviceGray | DeviceRGB | None = None,
        fill_color: int | tuple[Number, Number, Number] | DeviceGray | DeviceRGB | None = None,
    ) -> None: ...

    replace = dataclasses.replace

    @overload
    @staticmethod
    def combine(default_style: None, override_style: None) -> None: ...  # type: ignore[misc]
    @overload
    @staticmethod
    def combine(default_style: FontFace | None, override_style: FontFace | None) -> FontFace: ...

class TextStyle(FontFace):
    t_margin: int
    l_margin: int | Align
    b_margin: int
    def __init__(
        self,
        font_family: str | None = None,
        font_style: str | None = None,
        font_size_pt: int | None = None,
        color: int | tuple[int, int, int] | None = None,
        fill_color: int | tuple[int, int, int] | None = None,
        underline: bool = False,
        t_margin: int | None = None,
        l_margin: int | Align | str | None = None,
        b_margin: int | None = None,
    ): ...
    def replace(  # type: ignore[override]
        self,
        /,
        font_family: str | None = None,
        emphasis: TextEmphasis | None = None,
        font_size_pt: int | None = None,
        color: int | tuple[int, int, int] | None = None,
        fill_color: int | tuple[int, int, int] | None = None,
        t_margin: int | None = None,
        l_margin: int | None = None,
        b_margin: int | None = None,
    ) -> TextStyle: ...

@deprecated("fpdf.TitleStyle is deprecated since 2.7.10. It has been replaced by fpdf.TextStyle.")
class TitleStyle(TextStyle): ...

__pdoc__: Final[dict[str, bool]]

class CoreFont:
    __slots__ = ("i", "type", "name", "sp", "ss", "up", "ut", "cw", "fontkey", "emphasis")
    i: int
    type: str
    name: str
    up: int
    ut: int
    sp: int
    ss: int
    cw: int
    fontkey: str
    emphasis: TextEmphasis
    def __init__(self, fpdf, fontkey: str, style: int) -> None: ...
    def get_text_width(self, text: str, font_size_pt: int, _: Unused) -> float: ...
    def encode_text(self, text: str) -> str: ...

class TTFFont:
    __slots__ = (
        "i",
        "type",
        "name",
        "desc",
        "glyph_ids",
        "hbfont",
        "sp",
        "ss",
        "up",
        "ut",
        "cw",
        "ttffile",
        "fontkey",
        "emphasis",
        "scale",
        "subset",
        "cmap",
        "ttfont",
        "missing_glyphs",
    )
    i: int
    type: str
    ttffile: Incomplete
    fontkey: str
    ttfont: _TTFont
    scale: float
    desc: PDFFontDescriptor
    cw: defaultdict[str, int]
    cmap: Incomplete
    glyph_ids: dict[Incomplete, Incomplete]
    missing_glyphs: list[Incomplete]
    name: str
    up: int
    ut: int
    sp: int
    ss: int
    emphasis: TextEmphasis
    subset: SubsetMap
    hbfont: HarfBuzzFont | None  # Not always defined.
    def __init__(self, fpdf, font_file_path, fontkey: str, style: int) -> None: ...
    def __deepcopy__(self, memo) -> Self: ...
    def close(self) -> None: ...
    def get_text_width(self, text: str, font_size_pt: int, text_shaping_params): ...
    def shaped_text_width(self, text: str, font_size_pt: int, text_shaping_params): ...
    def perform_harfbuzz_shaping(self, text: str, font_size_pt: int, text_shaping_params): ...
    def encode_text(self, text: str) -> str: ...
    def shape_text(self, text: str, font_size_pt: int, text_shaping_params): ...

class PDFFontDescriptor(PDFObject):
    type: Incomplete
    ascent: Incomplete
    descent: Incomplete
    cap_height: Incomplete
    flags: Incomplete
    font_b_box: Incomplete
    italic_angle: Incomplete
    stem_v: Incomplete
    missing_width: Incomplete
    font_name: Incomplete
    def __init__(self, ascent, descent, cap_height, flags, font_b_box, italic_angle, stem_v, missing_width) -> None: ...

@dataclass(order=True)
class Glyph:
    __slots__ = ("glyph_id", "unicode", "glyph_name", "glyph_width")
    glyph_id: int
    unicode: tuple[Incomplete, ...]
    glyph_name: str
    glyph_width: int
    def __hash__(self) -> int: ...

class SubsetMap:
    font: TTFFont
    def __init__(self, font: TTFFont) -> None: ...
    def __len__(self) -> int: ...
    def items(self) -> Generator[Incomplete, None, None]: ...
    def pick(self, unicode: int): ...
    def pick_glyph(self, glyph): ...
    def get_glyph(self, glyph=None, unicode=None, glyph_name=None, glyph_width=None) -> Glyph: ...
    def get_all_glyph_names(self): ...

CORE_FONTS: dict[str, str]
COURIER_FONT: dict[str, int]
CORE_FONTS_CHARWIDTHS: dict[str, dict[str, int]]



import abc
from _typeshed import Incomplete, Unused
from abc import ABC, ABCMeta, abstractmethod
from builtins import bool as _bool
from collections.abc import Callable, Generator, Iterable, Iterator, Sequence
from contextlib import contextmanager
from enum import Enum
from types import TracebackType
from typing import Any, Generic, Literal, TypeVar, overload
from typing_extensions import ParamSpec, Self

from google.protobuf.message import Message
from tensorflow import (
    data as data,
    experimental as experimental,
    feature_column as feature_column,
    initializers as initializers,
    io as io,
    keras as keras,
    math as math,
    random as random,
    types as types,
)
from tensorflow._aliases import (
    AnyArray,
    DTypeLike,
    IntArray,
    ScalarTensorCompatible,
    ShapeLike,
    Slice,
    SparseTensorCompatible,
    TensorCompatible,
    UIntTensorCompatible,
)
from tensorflow.autodiff import GradientTape as GradientTape
from tensorflow.core.protobuf import struct_pb2
from tensorflow.dtypes import *
from tensorflow.experimental.dtensor import Layout
from tensorflow.keras import losses as losses
from tensorflow.linalg import eye as eye

# Most tf.math functions are exported as tf, but sadly not all are.
from tensorflow.math import (
    abs as abs,
    add as add,
    add_n as add_n,
    argmax as argmax,
    argmin as argmin,
    cos as cos,
    cosh as cosh,
    divide as divide,
    equal as equal,
    greater as greater,
    greater_equal as greater_equal,
    less as less,
    less_equal as less_equal,
    logical_and as logical_and,
    logical_not as logical_not,
    logical_or as logical_or,
    maximum as maximum,
    minimum as minimum,
    multiply as multiply,
    not_equal as not_equal,
    pow as pow,
    reduce_max as reduce_max,
    reduce_mean as reduce_mean,
    reduce_min as reduce_min,
    reduce_prod as reduce_prod,
    reduce_sum as reduce_sum,
    round as round,
    sigmoid as sigmoid,
    sign as sign,
    sin as sin,
    sinh as sinh,
    sqrt as sqrt,
    square as square,
    subtract as subtract,
    tanh as tanh,
)
from tensorflow.python.trackable.autotrackable import AutoTrackable
from tensorflow.sparse import SparseTensor as SparseTensor

# Tensors ideally should be a generic type, but properly typing data type/shape
# will be a lot of work. Until we have good non-generic tensorflow stubs,
# we will skip making Tensor generic. Also good type hints for shapes will
# run quickly into many places where type system is not strong enough today.
# So shape typing is probably not worth doing anytime soon.
class Tensor:
    def __init__(self, op: Operation, value_index: int, dtype: DType) -> None: ...
    def consumers(self) -> list[Incomplete]: ...
    @property
    def shape(self) -> TensorShape: ...
    def get_shape(self) -> TensorShape: ...
    @property
    def dtype(self) -> DType: ...
    @property
    def graph(self) -> Graph: ...
    @property
    def name(self) -> str: ...
    @property
    def op(self) -> Operation: ...
    def numpy(self) -> AnyArray: ...
    def __array__(self, dtype: DTypeLike | None = None) -> AnyArray: ...
    def __int__(self) -> int: ...
    def __abs__(self, name: str | None = None) -> Tensor: ...
    def __add__(self, other: TensorCompatible) -> Tensor: ...
    def __radd__(self, other: TensorCompatible) -> Tensor: ...
    def __sub__(self, other: TensorCompatible) -> Tensor: ...
    def __rsub__(self, other: TensorCompatible) -> Tensor: ...
    def __mul__(self, other: TensorCompatible) -> Tensor: ...
    def __rmul__(self, other: TensorCompatible) -> Tensor: ...
    def __pow__(self, other: TensorCompatible) -> Tensor: ...
    def __matmul__(self, other: TensorCompatible) -> Tensor: ...
    def __rmatmul__(self, other: TensorCompatible) -> Tensor: ...
    def __floordiv__(self, other: TensorCompatible) -> Tensor: ...
    def __rfloordiv__(self, other: TensorCompatible) -> Tensor: ...
    def __truediv__(self, other: TensorCompatible) -> Tensor: ...
    def __rtruediv__(self, other: TensorCompatible) -> Tensor: ...
    def __neg__(self, name: str | None = None) -> Tensor: ...
    def __and__(self, other: TensorCompatible) -> Tensor: ...
    def __rand__(self, other: TensorCompatible) -> Tensor: ...
    def __or__(self, other: TensorCompatible) -> Tensor: ...
    def __ror__(self, other: TensorCompatible) -> Tensor: ...
    def __eq__(self, other: TensorCompatible) -> Tensor: ...  # type: ignore[override]
    def __ne__(self, other: TensorCompatible) -> Tensor: ...  # type: ignore[override]
    def __ge__(self, other: TensorCompatible, name: str | None = None) -> Tensor: ...
    def __gt__(self, other: TensorCompatible, name: str | None = None) -> Tensor: ...
    def __le__(self, other: TensorCompatible, name: str | None = None) -> Tensor: ...
    def __lt__(self, other: TensorCompatible, name: str | None = None) -> Tensor: ...
    def __bool__(self) -> _bool: ...
    def __getitem__(self, slice_spec: Slice | tuple[Slice, ...]) -> Tensor: ...
    def __len__(self) -> int: ...
    # This only works for rank 0 tensors.
    def __index__(self) -> int: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class VariableSynchronization(Enum):
    AUTO = 0
    NONE = 1
    ON_WRITE = 2
    ON_READ = 3

class VariableAggregation(Enum):
    NONE = 0
    SUM = 1
    MEAN = 2
    ONLY_FIRST_REPLICA = 3

class _VariableMetaclass(type): ...

# Variable class in intent/documentation is a Tensor. In implementation there's
# TODO: comment to make it Tensor. It is not actually Tensor type wise, but even
# dynamically patches on most methods of tf.Tensor
# https://github.com/tensorflow/tensorflow/blob/9524a636cae9ae3f0554203c1ba7ee29c85fcf12/tensorflow/python/ops/variables.py#L1086.
class Variable(Tensor, metaclass=_VariableMetaclass):
    def __init__(
        self,
        initial_value: Tensor | Callable[[], Tensor] | None = None,
        trainable: _bool | None = None,
        validate_shape: _bool = True,
        # Valid non-None values are deprecated.
        caching_device: None = None,
        name: str | None = None,
        # Real type is VariableDef protobuf type. Can be added after adding script
        # to generate tensorflow protobuf stubs with mypy-protobuf.
        variable_def=None,
        dtype: DTypeLike | None = None,
        import_scope: str | None = None,
        constraint: Callable[[Tensor], Tensor] | None = None,
        synchronization: VariableSynchronization = ...,
        aggregation: VariableAggregation = ...,
        shape: ShapeLike | None = None,
        experimental_enable_variable_lifting: _bool = True,
    ) -> None: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class RaggedTensor(metaclass=ABCMeta):
    def bounding_shape(
        self, axis: TensorCompatible | None = None, name: str | None = None, out_type: DTypeLike | None = None
    ) -> Tensor: ...
    @classmethod
    def from_sparse(cls, st_input: SparseTensor, name: str | None = None, row_splits_dtype: DTypeLike = ...) -> RaggedTensor: ...
    def to_sparse(self, name: str | None = None) -> SparseTensor: ...
    def to_tensor(
        self, default_value: float | str | None = None, name: str | None = None, shape: ShapeLike | None = None
    ) -> Tensor: ...
    def __add__(self, other: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
    def __radd__(self, other: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
    def __sub__(self, other: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
    def __mul__(self, other: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
    def __rmul__(self, other: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
    def __floordiv__(self, other: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
    def __truediv__(self, other: RaggedTensor | float, name: str | None = None) -> RaggedTensor: ...
    def __getitem__(self, slice_spec: Slice | tuple[Slice, ...]) -> RaggedTensor: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class Operation:
    def __init__(
        self,
        node_def,
        g: Graph,
        # isinstance is used so can not be Sequence/Iterable.
        inputs: list[Tensor] | None = None,
        output_types: Unused = None,
        control_inputs: Iterable[Tensor | Operation] | None = None,
        input_types: Iterable[DType] | None = None,
        original_op: Operation | None = None,
        op_def=None,
    ) -> None: ...
    @property
    def inputs(self) -> list[Tensor]: ...
    @property
    def outputs(self) -> list[Tensor]: ...
    @property
    def device(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def type(self) -> str: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class TensorShape(metaclass=ABCMeta):
    def __init__(self, dims: ShapeLike) -> None: ...
    @property
    def rank(self) -> int: ...
    def as_list(self) -> list[int | None]: ...
    def assert_has_rank(self, rank: int) -> None: ...
    def assert_is_compatible_with(self, other: Iterable[int | None]) -> None: ...
    def __bool__(self) -> _bool: ...
    @overload
    def __getitem__(self, key: int) -> int | None: ...
    @overload
    def __getitem__(self, key: slice) -> TensorShape: ...
    def __iter__(self) -> Iterator[int | None]: ...
    def __len__(self) -> int: ...
    def __add__(self, other: Iterable[int | None]) -> TensorShape: ...
    def __radd__(self, other: Iterable[int | None]) -> TensorShape: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class Graph:
    def add_to_collection(self, name: str, value: object) -> None: ...
    def add_to_collections(self, names: Iterable[str] | str, value: object) -> None: ...
    @contextmanager
    def as_default(self) -> Generator[Self]: ...
    def finalize(self) -> None: ...
    def get_tensor_by_name(self, name: str) -> Tensor: ...
    def get_operation_by_name(self, name: str) -> Operation: ...
    def get_operations(self) -> list[Operation]: ...
    def get_name_scope(self) -> str: ...
    def __getattr__(self, name: str) -> Incomplete: ...

class IndexedSlices(metaclass=ABCMeta):
    def __init__(self, values: Tensor, indices: Tensor, dense_shape: None | Tensor = None) -> None: ...
    @property
    def values(self) -> Tensor: ...
    @property
    def indices(self) -> Tensor: ...
    @property
    def dense_shape(self) -> None | Tensor: ...
    @property
    def shape(self) -> TensorShape: ...
    @property
    def dtype(self) -> DType: ...
    @property
    def name(self) -> str: ...
    @property
    def op(self) -> Operation: ...
    @property
    def graph(self) -> Graph: ...
    @property
    def device(self) -> str: ...
    def __neg__(self) -> IndexedSlices: ...
    def consumers(self) -> list[Operation]: ...

class name_scope(metaclass=abc.ABCMeta):
    def __init__(self, name: str) -> None: ...
    def __enter__(self) -> str: ...
    def __exit__(self, typ: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None) -> None: ...

_P = ParamSpec("_P")
_R = TypeVar("_R")

class Module(AutoTrackable):
    def __init__(self, name: str | None = None) -> None: ...
    @property
    def name(self) -> str: ...
    @property
    def name_scope(self) -> name_scope: ...
    # Documentation only specifies these as returning Sequence. Actual
    # implementation does tuple.
    @property
    def variables(self) -> Sequence[Variable]: ...
    @property
    def trainable_variables(self) -> Sequence[Variable]: ...
    @property
    def non_trainable_variables(self) -> Sequence[Variable]: ...
    @property
    def submodules(self) -> Sequence[Module]: ...
    @classmethod
    def with_name_scope(cls, method: Callable[_P, _R]) -> Callable[_P, _R]: ...

class UnconnectedGradients(Enum):
    NONE = "none"
    ZERO = "zero"

_SpecProto = TypeVar("_SpecProto", bound=Message)

class TypeSpec(ABC, Generic[_SpecProto]):
    @property
    @abstractmethod
    def value_type(self) -> Any: ...
    def experimental_as_proto(self) -> _SpecProto: ...
    @classmethod
    def experimental_from_proto(cls, proto: _SpecProto) -> Self: ...
    @classmethod
    def experimental_type_proto(cls) -> type[_SpecProto]: ...
    def is_compatible_with(self, spec_or_value: Self | TensorCompatible | SparseTensor | RaggedTensor) -> _bool: ...
    # Incomplete as tf.types is not yet covered.
    def is_subtype_of(self, other) -> _bool: ...
    def most_specific_common_supertype(self, others: Sequence[Incomplete]) -> Self | None: ...
    def most_specific_compatible_type(self, other: Self) -> Self: ...

class TensorSpec(TypeSpec[struct_pb2.TensorSpecProto]):
    def __init__(self, shape: ShapeLike, dtype: DTypeLike = ..., name: str | None = None) -> None: ...
    @property
    def value_type(self) -> Tensor: ...
    @property
    def shape(self) -> TensorShape: ...
    @property
    def dtype(self) -> DType: ...
    @property
    def name(self) -> str | None: ...
    @classmethod
    def from_spec(cls, spec: TypeSpec[Any], name: str | None = None) -> Self: ...
    @classmethod
    def from_tensor(cls, tensor: Tensor, name: str | None = None) -> Self: ...
    def is_compatible_with(self, spec_or_tensor: Self | TensorCompatible) -> _bool: ...  # type: ignore[override]

class SparseTensorSpec(TypeSpec[struct_pb2.TypeSpecProto]):
    def __init__(self, shape: ShapeLike | None = None, dtype: DTypeLike = ...) -> None: ...
    @property
    def value_type(self) -> SparseTensor: ...
    @property
    def shape(self) -> TensorShape: ...
    @property
    def dtype(self) -> DType: ...
    @classmethod
    def from_value(cls, value: SparseTensor) -> Self: ...

class RaggedTensorSpec(TypeSpec[struct_pb2.TypeSpecProto]):
    def __init__(
        self,
        shape: ShapeLike | None = None,
        dtype: DTypeLike = ...,
        ragged_rank: int | None = None,
        row_splits_dtype: DTypeLike = ...,
        flat_values_spec: TypeSpec[Any] | None = None,
    ) -> None: ...
    @property
    def value_type(self) -> RaggedTensor: ...
    @property
    def shape(self) -> TensorShape: ...
    @property
    def dtype(self) -> DType: ...
    @classmethod
    def from_value(cls, value: RaggedTensor) -> Self: ...

def convert_to_tensor(
    value: TensorCompatible | IndexedSlices,
    dtype: DTypeLike | None = None,
    dtype_hint: DTypeLike | None = None,
    name: str | None = None,
) -> Tensor: ...
@overload
def expand_dims(input: TensorCompatible, axis: int, name: str | None = None) -> Tensor: ...
@overload
def expand_dims(input: RaggedTensor, axis: int, name: str | None = None) -> RaggedTensor: ...
@overload
def concat(values: TensorCompatible, axis: int, name: str | None = "concat") -> Tensor: ...
@overload
def concat(values: Sequence[RaggedTensor], axis: int, name: str | None = "concat") -> RaggedTensor: ...
@overload
def squeeze(
    input: TensorCompatible, axis: int | tuple[int, ...] | list[int] | None = None, name: str | None = None
) -> Tensor: ...
@overload
def squeeze(input: RaggedTensor, axis: int | tuple[int, ...] | list[int], name: str | None = None) -> RaggedTensor: ...
def tensor_scatter_nd_update(
    tensor: TensorCompatible, indices: TensorCompatible, updates: TensorCompatible, name: str | None = None
) -> Tensor: ...
def constant(
    value: TensorCompatible, dtype: DTypeLike | None = None, shape: ShapeLike | None = None, name: str | None = "Const"
) -> Tensor: ...
@overload
def cast(x: TensorCompatible, dtype: DTypeLike, name: str | None = None) -> Tensor: ...
@overload
def cast(x: SparseTensor, dtype: DTypeLike, name: str | None = None) -> SparseTensor: ...
@overload
def cast(x: RaggedTensor, dtype: DTypeLike, name: str | None = None) -> RaggedTensor: ...
def zeros(shape: ShapeLike, dtype: DTypeLike = ..., name: str | None = None, layout: Layout | None = None) -> Tensor: ...
def ones(shape: ShapeLike, dtype: DTypeLike = ..., name: str | None = None, layout: Layout | None = None) -> Tensor: ...
@overload
def zeros_like(
    input: TensorCompatible | IndexedSlices, dtype: DTypeLike | None = None, name: str | None = None, layout: Layout | None = None
) -> Tensor: ...
@overload
def zeros_like(
    input: RaggedTensor, dtype: DTypeLike | None = None, name: str | None = None, layout: Layout | None = None
) -> RaggedTensor: ...
@overload
def ones_like(
    input: TensorCompatible, dtype: DTypeLike | None = None, name: str | None = None, layout: Layout | None = None
) -> Tensor: ...
@overload
def ones_like(
    input: RaggedTensor, dtype: DTypeLike | None = None, name: str | None = None, layout: Layout | None = None
) -> RaggedTensor: ...
def reshape(tensor: TensorCompatible, shape: ShapeLike | Tensor, name: str | None = None) -> Tensor: ...
def pad(
    tensor: TensorCompatible,
    paddings: Tensor | IntArray | Iterable[Iterable[int]],
    mode: Literal["CONSTANT", "constant", "REFLECT", "reflect", "SYMMETRIC", "symmetric"] = "CONSTANT",
    constant_values: ScalarTensorCompatible = 0,
    name: str | None = None,
) -> Tensor: ...
def shape(input: SparseTensorCompatible, out_type: DTypeLike | None = None, name: str | None = None) -> Tensor: ...
def where(
    condition: TensorCompatible, x: TensorCompatible | None = None, y: TensorCompatible | None = None, name: str | None = None
) -> Tensor: ...
def gather_nd(
    params: TensorCompatible,
    indices: UIntTensorCompatible,
    batch_dims: UIntTensorCompatible = 0,
    name: str | None = None,
    bad_indices_policy: Literal["", "DEFAULT", "ERROR", "IGNORE"] = "",
) -> Tensor: ...
def __getattr__(name: str): ...  # incomplete module



from typing import Final

XK_3270_Duplicate: Final = 0xFD01
XK_3270_FieldMark: Final = 0xFD02
XK_3270_Right2: Final = 0xFD03
XK_3270_Left2: Final = 0xFD04
XK_3270_BackTab: Final = 0xFD05
XK_3270_EraseEOF: Final = 0xFD06
XK_3270_EraseInput: Final = 0xFD07
XK_3270_Reset: Final = 0xFD08
XK_3270_Quit: Final = 0xFD09
XK_3270_PA1: Final = 0xFD0A
XK_3270_PA2: Final = 0xFD0B
XK_3270_PA3: Final = 0xFD0C
XK_3270_Test: Final = 0xFD0D
XK_3270_Attn: Final = 0xFD0E
XK_3270_CursorBlink: Final = 0xFD0F
XK_3270_AltCursor: Final = 0xFD10
XK_3270_KeyClick: Final = 0xFD11
XK_3270_Jump: Final = 0xFD12
XK_3270_Ident: Final = 0xFD13
XK_3270_Rule: Final = 0xFD14
XK_3270_Copy: Final = 0xFD15
XK_3270_Play: Final = 0xFD16
XK_3270_Setup: Final = 0xFD17
XK_3270_Record: Final = 0xFD18
XK_3270_ChangeScreen: Final = 0xFD19
XK_3270_DeleteWord: Final = 0xFD1A
XK_3270_ExSelect: Final = 0xFD1B
XK_3270_CursorSelect: Final = 0xFD1C
XK_3270_PrintScreen: Final = 0xFD1D
XK_3270_Enter: Final = 0xFD1E



from collections.abc import Callable, Mapping, Sequence
from typing import Any, Literal
from typing_extensions import TypeAlias

import tensorflow as tf
from requests.api import _HeadersMapping
from tensorflow.keras import Model
from tensorflow.keras.optimizers.schedules import LearningRateSchedule

_Logs: TypeAlias = Mapping[str, Any] | None | Any

class Callback:
    params: dict[str, Any]
    def set_params(self, params: dict[str, Any]) -> None: ...
    def set_model(self, model: Model[Any, Any]) -> None: ...
    @property
    def model(self) -> Model[Any, Any]: ...
    def on_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_epoch_begin(self, epoch: int, logs: _Logs = None) -> None: ...
    def on_epoch_end(self, epoch: int, logs: _Logs = None) -> None: ...
    def on_train_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_train_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_test_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_test_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_predict_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_predict_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_train_begin(self, logs: _Logs = None) -> None: ...
    def on_train_end(self, logs: _Logs = None) -> None: ...
    def on_test_begin(self, logs: _Logs = None) -> None: ...
    def on_test_end(self, logs: _Logs = None) -> None: ...
    def on_predict_begin(self, logs: _Logs = None) -> None: ...
    def on_predict_end(self, logs: _Logs = None) -> None: ...

class CallbackList(Callback):
    def __init__(
        self,
        callbacks: Sequence[Callback] | None = None,
        add_history: bool = False,
        add_progbar: bool = False,
        # model: Model[Any, object] | None = None,
        model: Model[Any, Any] | None = None,
        **params: Any,
    ) -> None: ...
    def set_params(self, params: dict[str, Any]) -> None: ...
    def set_model(self, model: Model[Any, Any]) -> None: ...
    def on_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_epoch_begin(self, epoch: int, logs: _Logs = None) -> None: ...
    def on_epoch_end(self, epoch: int, logs: _Logs = None) -> None: ...
    def on_train_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_train_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_test_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_test_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_predict_batch_begin(self, batch: int, logs: _Logs = None) -> None: ...
    def on_predict_batch_end(self, batch: int, logs: _Logs = None) -> None: ...
    def on_train_begin(self, logs: _Logs = None) -> None: ...
    def on_train_end(self, logs: _Logs = None) -> None: ...
    def on_test_begin(self, logs: _Logs = None) -> None: ...
    def on_test_end(self, logs: _Logs = None) -> None: ...
    def on_predict_begin(self, logs: _Logs = None) -> None: ...
    def on_predict_end(self, logs: _Logs = None) -> None: ...

class BackupAndRestore(Callback):
    def __init__(self, backup_dir: str, save_freq: str = "epoch", delete_checkpoint: bool = True) -> None: ...

class CSVLogger(Callback):
    def __init__(self, filename: str, separator: str = ",", append: bool = False) -> None: ...

class EarlyStopping(Callback):
    monitor_op: Any
    def __init__(
        self,
        monitor: str = "val_loss",
        min_delta: float = 0,
        patience: int = 0,
        verbose: Literal[0, 1] = 0,
        mode: Literal["auto", "min", "max"] = "auto",
        baseline: float | None = None,
        restore_best_weights: bool = False,
        start_from_epoch: int = 0,
    ) -> None: ...

class History(Callback):
    history: dict[str, list[Any]]

class LambdaCallback(Callback):
    def __init__(
        self,
        on_epoch_begin: Callable[[int, _Logs], object] | None = None,
        on_epoch_end: Callable[[int, _Logs], object] | None = None,
        on_train_begin: Callable[[_Logs], object] | None = None,
        on_train_end: Callable[[_Logs], object] | None = None,
        on_train_batch_begin: Callable[[int, _Logs], object] | None = None,
        on_train_batch_end: Callable[[int, _Logs], object] | None = None,
        **kwargs: Any,
    ) -> None: ...

class LearningRateScheduler(Callback):
    def __init__(
        self,
        schedule: LearningRateSchedule | Callable[[int], float | tf.Tensor] | Callable[[int, float], float | tf.Tensor],
        verbose: Literal[0, 1] = 0,
    ) -> None: ...

class ModelCheckpoint(Callback):
    monitor_op: Any
    filepath: str
    def __init__(
        self,
        filepath: str,
        monitor: str = "val_loss",
        verbose: Literal[0, 1] = 0,
        save_best_only: bool = False,
        save_weights_only: bool = False,
        mode: Literal["auto", "min", "max"] = "auto",
        save_freq: str | int = "epoch",
        initial_value_threshold: float | None = None,
    ) -> None: ...
    def _save_model(self, epoch: int, batch: int | None, logs: _Logs) -> None: ...

class ProgbarLogger(Callback):
    use_steps: bool
    def __init__(self) -> None: ...

class ReduceLROnPlateau(Callback):
    def __init__(
        self,
        monitor: str = "val_loss",
        factor: float = 0.1,
        patience: int = 10,
        verbose: Literal[0, 1] = 0,
        mode: Literal["auto", "min", "max"] = "auto",
        min_delta: float = 1e-4,
        cooldown: int = 0,
        min_lr: float = 0.0,
        **kwargs,
    ) -> None: ...
    def in_cooldown(self) -> bool: ...

class RemoteMonitor(Callback):
    def __init__(
        self,
        root: str = "http://localhost:9000",
        path: str = "/publish/epoch/end/",
        field: str = "data",
        headers: _HeadersMapping | None = None,
        send_as_json: bool = False,
    ) -> None: ...

class TensorBoard(Callback):
    write_steps_per_second: bool
    update_freq: int | Literal["epoch"]

    def __init__(
        self,
        log_dir: str = "logs",
        histogram_freq: int = 0,
        write_graph: bool = True,
        write_images: bool = False,
        write_steps_per_second: bool = False,
        update_freq: int | Literal["epoch"] = "epoch",
        profile_batch: int | tuple[int, int] = 0,
        embeddings_freq: int = 0,
        embeddings_metadata: dict[str, None] | None = None,
    ) -> None: ...
    def _write_keras_model_train_graph(self) -> None: ...
    def _stop_trace(self, batch: int | None = None) -> None: ...

class TerminateOnNaN(Callback): ...



from collections.abc import Callable
from typing import Any
from typing_extensions import Self

__tracebackhide__: bool

class CollectionMixin:
    def is_iterable(self) -> Self: ...
    def is_not_iterable(self) -> Self: ...
    def is_subset_of(self, *supersets: Any) -> Self: ...
    def is_sorted(self, key: Callable[[Any], Any] = ..., reverse: bool = False) -> Self: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = [
    "current_flow_betweenness_centrality",
    "approximate_current_flow_betweenness_centrality",
    "edge_current_flow_betweenness_centrality",
]

@_dispatchable
def approximate_current_flow_betweenness_centrality(
    G: Graph[_Node],
    normalized: bool | None = True,
    weight: str | None = None,
    dtype: type = ...,
    solver: str = "full",
    epsilon: float = 0.5,
    kmax: int = 10000,
    seed: int | RandomState | None = None,
) -> dict[Incomplete, float]: ...
@_dispatchable
def current_flow_betweenness_centrality(
    G: Graph[_Node], normalized: bool | None = True, weight: str | None = None, dtype: type = ..., solver: str = "full"
) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def edge_current_flow_betweenness_centrality(
    G: Graph[_Node], normalized: bool | None = True, weight: str | None = None, dtype: type = ..., solver: str = "full"
) -> dict[tuple[Incomplete, Incomplete], float]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol messages for describing the configuration of the ExampleParserOp."""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class VarLenFeatureProto(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DTYPE_FIELD_NUMBER: builtins.int
    VALUES_OUTPUT_TENSOR_NAME_FIELD_NUMBER: builtins.int
    INDICES_OUTPUT_TENSOR_NAME_FIELD_NUMBER: builtins.int
    SHAPES_OUTPUT_TENSOR_NAME_FIELD_NUMBER: builtins.int
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    values_output_tensor_name: builtins.str
    indices_output_tensor_name: builtins.str
    shapes_output_tensor_name: builtins.str
    def __init__(
        self,
        *,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        values_output_tensor_name: builtins.str | None = ...,
        indices_output_tensor_name: builtins.str | None = ...,
        shapes_output_tensor_name: builtins.str | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "dtype",
            b"dtype",
            "indices_output_tensor_name",
            b"indices_output_tensor_name",
            "shapes_output_tensor_name",
            b"shapes_output_tensor_name",
            "values_output_tensor_name",
            b"values_output_tensor_name",
        ],
    ) -> None: ...

global___VarLenFeatureProto = VarLenFeatureProto

@typing.final
class FixedLenFeatureProto(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DTYPE_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    DEFAULT_VALUE_FIELD_NUMBER: builtins.int
    VALUES_OUTPUT_TENSOR_NAME_FIELD_NUMBER: builtins.int
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    values_output_tensor_name: builtins.str
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    @property
    def default_value(self) -> tensorflow.core.framework.tensor_pb2.TensorProto: ...
    def __init__(
        self,
        *,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        default_value: tensorflow.core.framework.tensor_pb2.TensorProto | None = ...,
        values_output_tensor_name: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["default_value", b"default_value", "shape", b"shape"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "default_value",
            b"default_value",
            "dtype",
            b"dtype",
            "shape",
            b"shape",
            "values_output_tensor_name",
            b"values_output_tensor_name",
        ],
    ) -> None: ...

global___FixedLenFeatureProto = FixedLenFeatureProto

@typing.final
class FeatureConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FIXED_LEN_FEATURE_FIELD_NUMBER: builtins.int
    VAR_LEN_FEATURE_FIELD_NUMBER: builtins.int
    @property
    def fixed_len_feature(self) -> global___FixedLenFeatureProto: ...
    @property
    def var_len_feature(self) -> global___VarLenFeatureProto: ...
    def __init__(
        self,
        *,
        fixed_len_feature: global___FixedLenFeatureProto | None = ...,
        var_len_feature: global___VarLenFeatureProto | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "config", b"config", "fixed_len_feature", b"fixed_len_feature", "var_len_feature", b"var_len_feature"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "config", b"config", "fixed_len_feature", b"fixed_len_feature", "var_len_feature", b"var_len_feature"
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["config", b"config"]
    ) -> typing.Literal["fixed_len_feature", "var_len_feature"] | None: ...

global___FeatureConfiguration = FeatureConfiguration

@typing.final
class ExampleParserConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class FeatureMapEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___FeatureConfiguration: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___FeatureConfiguration | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    FEATURE_MAP_FIELD_NUMBER: builtins.int
    @property
    def feature_map(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___FeatureConfiguration]: ...
    def __init__(
        self, *, feature_map: collections.abc.Mapping[builtins.str, global___FeatureConfiguration] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["feature_map", b"feature_map"]) -> None: ...

global___ExampleParserConfiguration = ExampleParserConfiguration



from typing import ClassVar, Final, TypeVar

from docutils import readers

_S = TypeVar("_S", bound=str | bytes)

__docformat__: Final = "reStructuredText"

class Reader(readers.ReReader[_S]):
    config_section_dependencies: ClassVar[tuple[str, ...]]



import io
from _typeshed import ReadableBuffer
from typing import Any, Literal, NoReturn
from typing_extensions import TypeAlias

cERROR_PIPE_BUSY: int
cSECURITY_SQOS_PRESENT: int
cSECURITY_ANONYMOUS: int
MAXIMUM_RETRY_COUNT: int

def check_closed(f): ...

_PyHANDLE: TypeAlias = Any  # pywin32._win32typing.PyHANDLE

class NpipeSocket:
    def __init__(self, handle: _PyHANDLE | None = None) -> None: ...
    def accept(self) -> None: ...
    def bind(self, address) -> None: ...
    def close(self) -> None: ...
    flags: int
    def connect(self, address: str, retry_count: int = 0) -> None: ...
    def connect_ex(self, address: str) -> None: ...
    def detach(self) -> _PyHANDLE | None: ...
    def dup(self) -> NpipeSocket: ...
    def getpeername(self) -> str: ...
    def getsockname(self) -> str: ...
    # NotImplementedError
    def getsockopt(self, level, optname, buflen=None) -> NoReturn: ...
    # NotImplementedError
    def ioctl(self, control, option) -> NoReturn: ...
    # NotImplementedError
    def listen(self, backlog) -> NoReturn: ...
    def makefile(self, mode: str | None = None, bufsize: int | None = None) -> io.BufferedReader: ...
    def recv(self, bufsize: int, flags: int = 0) -> str: ...
    def recvfrom(self, bufsize: int, flags: int = 0) -> tuple[str, str]: ...
    def recvfrom_into(self, buf: memoryview | ReadableBuffer, nbytes: int = 0, flags: int = 0) -> tuple[int, str]: ...
    def recv_into(self, buf: memoryview | ReadableBuffer, nbytes: int = 0) -> int: ...
    def send(self, string: str, flags: int = 0) -> int: ...
    def sendall(self, string: str, flags: int = 0) -> int: ...
    def sendto(self, string: str, address: str) -> int: ...
    def setblocking(self, flag: bool) -> None: ...
    def settimeout(self, value: float | None) -> None: ...
    def gettimeout(self) -> int | None: ...
    # NotImplementedError
    def setsockopt(self, level, optname, value) -> NoReturn: ...
    def shutdown(self, how) -> None: ...

class NpipeFileIOBase(io.RawIOBase):
    sock: NpipeSocket
    def __init__(self, npipe_socket: NpipeSocket) -> None: ...
    def close(self) -> None: ...
    def fileno(self): ...
    def isatty(self) -> Literal[False]: ...
    def readable(self) -> Literal[True]: ...
    def readinto(self, buf: memoryview | ReadableBuffer) -> int: ...
    def seekable(self) -> Literal[False]: ...
    def writable(self) -> Literal[False]: ...



EOF: None
E: dict[str, str]
namespaces: dict[str, str]
scopingElements: frozenset[tuple[str, str]]
formattingElements: frozenset[tuple[str, str]]
specialElements: frozenset[tuple[str, str]]
htmlIntegrationPointElements: frozenset[tuple[str, str]]
mathmlTextIntegrationPointElements: frozenset[tuple[str, str]]
adjustSVGAttributes: dict[str, str]
adjustMathMLAttributes: dict[str, str]
adjustForeignAttributes: dict[str, tuple[str, str, str] | tuple[None, str, str]]
unadjustForeignAttributes: dict[tuple[str, str], str]
spaceCharacters: frozenset[str]
tableInsertModeElements: frozenset[str]
asciiLowercase: frozenset[str]
asciiUppercase: frozenset[str]
asciiLetters: frozenset[str]
digits: frozenset[str]
hexDigits: frozenset[str]
asciiUpper2Lower: dict[int, int]
headingElements: tuple[str, ...]
voidElements: frozenset[str]
cdataElements: frozenset[str]
rcdataElements: frozenset[str]
booleanAttributes: dict[str, frozenset[str]]
entitiesWindows1252: tuple[int, ...]
xmlEntities: frozenset[str]
entities: dict[str, str]
replacementCharacters: dict[int, str]
tokenTypes: dict[str, int]
tagTokenTypes: frozenset[int]
prefixes: dict[str, str]

class DataLossWarning(UserWarning): ...
class _ReparseException(Exception): ...



from abc import ABCMeta
from collections.abc import Iterable
from typing import TypeVar

_T = TypeVar("_T")

class BasePathMixin:
    uri: str | None
    @property
    def absolute_uri(self) -> str: ...
    @property
    def base_path(self) -> str: ...
    @base_path.setter
    def base_path(self, newbase_path: str) -> None: ...
    def get_path_from_uri(self) -> str: ...

class GroupedBasePathMixin(Iterable[_T], metaclass=ABCMeta):
    @property
    def base_uri(self) -> str: ...
    @base_uri.setter
    def base_uri(self, __new_url: str, /) -> None: ...
    @property
    def base_path(self) -> str: ...
    @base_path.setter
    def base_path(self, __new_url: str, /) -> None: ...



import datetime
from typing import Final, Literal

from ..core import WesternCalendar

class Netherlands(WesternCalendar):
    include_carnival: bool
    def __init__(self, include_carnival: bool = False) -> None: ...
    def get_king_queen_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_carnival_days(self, year: int) -> list[tuple[datetime.date, str]]: ...

FALL_HOLIDAYS_EARLY_REGIONS: Final[dict[int, list[str]]]
SPRING_HOLIDAYS_EARLY_REGIONS: Final[dict[int, list[str]]]
SUMMER_HOLIDAYS_EARLY_REGIONS: Final[dict[int, list[str]]]
SUMMER_HOLIDAYS_LATE_REGIONS: Final[dict[int, list[str]]]

class NetherlandsWithSchoolHolidays(Netherlands):
    region: Literal["north", "middle", "south"]
    carnival_instead_of_spring: bool
    def __init__(
        self,
        region: Literal["north", "middle", "south"],
        carnival_instead_of_spring: bool = False,
        *,
        include_carnival: bool = ...,
    ) -> None: ...
    def get_fall_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...
    def get_christmas_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...
    def get_spring_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...
    def get_carnival_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...
    def get_may_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...
    def get_summer_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...



from typing import Literal

from tensorflow import Operation, Tensor
from tensorflow._aliases import TensorCompatible

def Fingerprint(data: TensorCompatible, method: Literal["farmhash64"], name: str | None = None) -> Tensor: ...
def Snapshot(input: TensorCompatible, name: str | None = None) -> Tensor: ...
def ResourceApplyAdagradV2(
    var: Tensor,
    accum: Tensor,
    lr: TensorCompatible,
    epsilon: TensorCompatible,
    grad: TensorCompatible,
    use_locking: bool = False,
    update_slots: bool = True,
    name: str | None = None,
) -> Operation: ...
def ResourceSparseApplyAdagradV2(
    var: Tensor,
    accum: Tensor,
    lr: TensorCompatible,
    epsilon: TensorCompatible,
    grad: TensorCompatible,
    indices: TensorCompatible,
    use_locking: bool = False,
    update_slots: bool = True,
    name: str | None = None,
) -> Operation: ...
def ResourceApplyAdam(
    var: Tensor,
    m: Tensor,
    v: Tensor,
    beta1_power: TensorCompatible,
    beta2_power: TensorCompatible,
    lr: TensorCompatible,
    beta1: TensorCompatible,
    beta2: TensorCompatible,
    epsilon: TensorCompatible,
    grad: TensorCompatible,
    use_locking: bool = False,
    use_nesterov: bool = False,
    name: str | None = None,
) -> Operation: ...
def __getattr__(name: str): ...  # incomplete module



import types
from _typeshed import Incomplete
from typing import ClassVar

from pony.orm import dbapiprovider, dbschema
from pony.orm.dbapiprovider import DBAPIProvider
from pony.orm.sqlbuilding import SQLBuilder, Value
from pony.orm.sqltranslation import SQLTranslator

NoneType: type[None]
mysql_module_name: str

class MySQLColumn(dbschema.Column):
    auto_template: ClassVar[str]

class MySQLSchema(dbschema.DBSchema):
    dialect: ClassVar[str]
    column_class: ClassVar[type[MySQLColumn]]

class MySQLTranslator(SQLTranslator):
    dialect: ClassVar[str]

class MySQLValue(Value):
    __slots__: list[str] = []

class MySQLBuilder(SQLBuilder):
    dialect: ClassVar[str]
    value_class: ClassVar[type[MySQLValue]]
    def CONCAT(builder, *args): ...
    def TRIM(builder, expr, chars=None): ...
    def LTRIM(builder, expr, chars=None): ...
    def RTRIM(builder, expr, chars=None): ...
    def TO_INT(builder, expr): ...
    def TO_REAL(builder, expr): ...
    def TO_STR(builder, expr): ...
    def YEAR(builder, expr): ...
    def MONTH(builder, expr): ...
    def DAY(builder, expr): ...
    def HOUR(builder, expr): ...
    def MINUTE(builder, expr): ...
    def SECOND(builder, expr): ...
    def DATE_ADD(builder, expr, delta): ...
    def DATE_SUB(builder, expr, delta): ...
    def DATE_DIFF(builder, expr1, expr2): ...
    def DATETIME_ADD(builder, expr, delta): ...
    def DATETIME_SUB(builder, expr, delta): ...
    def DATETIME_DIFF(builder, expr1, expr2): ...
    def JSON_QUERY(builder, expr, path): ...
    def JSON_VALUE(builder, expr, path, type): ...
    def JSON_NONZERO(builder, expr): ...
    def JSON_ARRAY_LENGTH(builder, value): ...
    def JSON_EQ(builder, left, right): ...
    def JSON_NE(builder, left, right): ...
    def JSON_CONTAINS(builder, expr, path, key): ...
    @classmethod
    def wrap_param_to_json_array(cls, values): ...
    def JSON_PARAM(builder, expr): ...

class MySQLStrConverter(dbapiprovider.StrConverter):
    def sql_type(converter): ...

class MySQLRealConverter(dbapiprovider.RealConverter):
    def sql_type(converter): ...

class MySQLBlobConverter(dbapiprovider.BlobConverter):
    def sql_type(converter): ...

class MySQLTimeConverter(dbapiprovider.TimeConverter):
    def sql2py(converter, val): ...

class MySQLTimedeltaConverter(dbapiprovider.TimedeltaConverter): ...

class MySQLUuidConverter(dbapiprovider.UuidConverter):
    def sql_type(converter): ...

class MySQLJsonConverter(dbapiprovider.JsonConverter):
    EQ: str
    NE: str
    def init(self, kwargs) -> None: ...

class MySQLProvider(DBAPIProvider):
    dialect: ClassVar[str]
    varchar_default_max_len: ClassVar[int]
    dbapi_module: ClassVar[types.ModuleType]
    dbschema_cls: ClassVar[type[MySQLSchema]]
    translator_cls: ClassVar[type[MySQLTranslator]]
    sqlbuilder_cls: ClassVar[type[MySQLBuilder]]
    fk_types: ClassVar[dict[str, str]]
    converter_classes: Incomplete
    def normalize_name(provider, name): ...
    def inspect_connection(provider, connection) -> None: ...
    def should_reconnect(provider, exc): ...
    def get_pool(provider, *args, **kwargs): ...
    def set_transaction_mode(provider, connection, cache) -> None: ...
    def release(provider, connection, cache=None) -> None: ...
    def table_exists(provider, connection, table_name, case_sensitive: bool = True): ...
    def index_exists(provider, connection, table_name, index_name, case_sensitive: bool = True): ...
    def fk_exists(provider, connection, table_name, fk_name, case_sensitive: bool = True): ...

provider_cls = MySQLProvider

def str2datetime(s): ...



import datetime
from typing import ClassVar

class SpringHoliday:
    include_spring_holiday: ClassVar[bool]

class SpringHolidayFirstMondayApril(SpringHoliday):
    def get_spring_holiday(self, year: int) -> tuple[datetime.date, str]: ...

class SpringHolidaySecondMondayApril(SpringHoliday):
    def get_spring_holiday(self, year: int) -> tuple[datetime.date, str]: ...

class SpringHolidayTuesdayAfterFirstMondayMay(SpringHoliday):
    def get_spring_holiday(self, year: int) -> tuple[datetime.date, str]: ...

class SpringHolidayLastMondayMay(SpringHoliday):
    def get_spring_holiday(self, year: int) -> tuple[datetime.date, str]: ...

class SpringHolidayFirstMondayJune(SpringHoliday):
    def get_spring_holiday(self, year: int) -> tuple[datetime.date, str]: ...



from __future__ import annotations

from typing_extensions import assert_type
from xml.dom.minidom import Document

document = Document()

assert_type(document.toxml(), str)
assert_type(document.toxml(encoding=None), str)
assert_type(document.toxml(encoding="UTF8"), bytes)
assert_type(document.toxml("UTF8"), bytes)
assert_type(document.toxml(standalone=True), str)
assert_type(document.toxml("UTF8", True), bytes)
assert_type(document.toxml(encoding="UTF8", standalone=True), bytes)


# Because toprettyxml can mix positional and keyword variants of the "encoding" argument, which
# determines the return type, the proper stub typing isn't immediately obvious. This is a basic
# brute-force sanity check.
# Test cases like toxml
assert_type(document.toprettyxml(), str)
assert_type(document.toprettyxml(encoding=None), str)
assert_type(document.toprettyxml(encoding="UTF8"), bytes)
assert_type(document.toprettyxml(standalone=True), str)
assert_type(document.toprettyxml(encoding="UTF8", standalone=True), bytes)
# Test cases unique to toprettyxml
assert_type(document.toprettyxml("  "), str)
assert_type(document.toprettyxml("  ", "\r\n"), str)
assert_type(document.toprettyxml("  ", "\r\n", "UTF8"), bytes)
assert_type(document.toprettyxml("  ", "\r\n", "UTF8", True), bytes)
assert_type(document.toprettyxml("  ", "\r\n", standalone=True), str)



from .jwe import JsonWebEncryption as JsonWebEncryption
from .models import (
    JWEAlgorithm as JWEAlgorithm,
    JWEAlgorithmWithTagAwareKeyAgreement as JWEAlgorithmWithTagAwareKeyAgreement,
    JWEEncAlgorithm as JWEEncAlgorithm,
    JWEZipAlgorithm as JWEZipAlgorithm,
)

__all__ = ["JsonWebEncryption", "JWEAlgorithm", "JWEAlgorithmWithTagAwareKeyAgreement", "JWEEncAlgorithm", "JWEZipAlgorithm"]



from _typeshed import Incomplete, StrPath
from typing import Literal, overload
from typing_extensions import TypeAlias

from docutils.core import Publisher
from docutils.nodes import document
from docutils.writers import _WriterParts

_HTMLHeaderLevel: TypeAlias = Literal[1, 2, 3, 4, 5, 6]

def html_parts(
    input_string: str | bytes,
    source_path: StrPath | None = None,
    destination_path: StrPath | None = None,
    input_encoding: str = "unicode",
    doctitle: bool = True,
    initial_header_level: _HTMLHeaderLevel = 1,
) -> _WriterParts: ...
@overload
def html_body(
    input_string: str | bytes,
    source_path: StrPath | None = None,
    destination_path: StrPath | None = None,
    input_encoding: str = "unicode",
    output_encoding: Literal["unicode"] = "unicode",
    doctitle: bool = True,
    initial_header_level: _HTMLHeaderLevel = 1,
) -> str: ...
@overload
def html_body(
    input_string: str | bytes,
    source_path: StrPath | None = None,
    destination_path: StrPath | None = None,
    input_encoding: str = "unicode",
    output_encoding: str = "unicode",
    doctitle: bool = True,
    initial_header_level: _HTMLHeaderLevel = 1,
) -> str | bytes: ...
def internals(
    source: str,
    source_path: StrPath | None = None,
    input_encoding: str = "unicode",
    settings_overrides: dict[str, Incomplete] | None = None,
) -> tuple[document | None, Publisher]: ...



from collections.abc import Hashable, Mapping
from typing import Literal, TypeVar, overload

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph
from networkx.classes.multidigraph import MultiDiGraph
from networkx.classes.multigraph import MultiGraph
from networkx.utils.backends import _dispatchable

_X = TypeVar("_X", bound=Hashable)
_Y = TypeVar("_Y", bound=Hashable)

__all__ = ["convert_node_labels_to_integers", "relabel_nodes"]

@overload
def relabel_nodes(G: MultiDiGraph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> MultiDiGraph[_X | _Y]: ...
@overload
def relabel_nodes(G: DiGraph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> DiGraph[_X | _Y]: ...
@overload
def relabel_nodes(G: MultiGraph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> MultiGraph[_X | _Y]: ...
@overload
def relabel_nodes(G: Graph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> Graph[_X | _Y]: ...
@_dispatchable
def convert_node_labels_to_integers(
    G: Graph[Hashable],
    first_label: int = 0,
    ordering: Literal["default", "sorted", "increasing degree", "decreasing degree"] = "default",
    label_attribute=None,
) -> Graph[int]: ...



from _typeshed import Unused
from _typeshed.wsgi import WSGIEnvironment
from collections import deque
from collections.abc import Callable, Mapping, Sequence
from logging import Logger
from threading import Condition, Lock

from .channel import HTTPChannel
from .utilities import Error

rename_headers: Mapping[str, str]
hop_by_hop: frozenset[str]

class ThreadedTaskDispatcher:
    stop_count: int
    active_count: int
    logger: Logger
    queue_logger: Logger
    threads: set[int]
    queue: deque[Task]
    lock: Lock
    queue_cv: Condition
    thread_exit_cv: Condition
    def start_new_thread(self, target: Callable[[int], Unused], thread_no: int) -> None: ...
    def handler_thread(self, thread_no: int) -> None: ...
    def set_thread_count(self, count: int) -> None: ...
    def add_task(self, task: Task) -> None: ...
    def shutdown(self, cancel_pending: bool = True, timeout: int = 5) -> bool: ...

class Task:
    close_on_finish: bool
    status: str
    wrote_header: bool
    start_time: int
    content_length: int | None
    content_bytes_written: int
    logged_write_excess: bool
    logged_write_no_body: bool
    complete: bool
    chunked_response: bool
    logger: Logger
    channel: HTTPChannel
    request: Error
    response_headers: Sequence[tuple[str, str]]
    version: str
    def __init__(self, channel: HTTPChannel, request: Error) -> None: ...
    def service(self) -> None: ...
    @property
    def has_body(self) -> bool: ...
    def set_close_on_finish(self) -> None: ...
    def build_response_header(self) -> bytes: ...
    def remove_content_length_header(self) -> None: ...
    def start(self) -> None: ...
    def finish(self) -> None: ...
    def write(self, data: bytes) -> None: ...

class ErrorTask(Task):
    complete: bool
    status: str
    close_on_finish: bool
    content_length: int | None
    def execute(self) -> None: ...

class WSGITask(Task):
    environ: WSGIEnvironment | None
    response_headers: Sequence[tuple[str, str]]
    complete: bool
    status: str
    content_length: int | None
    close_on_finish: bool
    def execute(self) -> None: ...
    def get_environment(self) -> WSGIEnvironment: ...



"""Jac library-mode surface area for Python codegen.

This module exists so the Jac compiler can import Jac "library mode" helpers
without importing `jaclang.lib` during bootstrap (which may require compiling
Jac code via the meta-importer).

It mirrors the behavior of `jaclang/lib.jac` and intentionally keeps the API
names stable because the Jac -> Python code generator emits imports for these.
"""

from __future__ import annotations

# Direct imports - no lazy loading needed since runtimelib is now pure Python
from jaclang.pycore.archetype import GenericEdge, Root
from jaclang.pycore.archetype import ObjectSpatialFunction as DSFunc
from jaclang.pycore.archetype import ObjectSpatialPath as OPath
from jaclang.pycore.constant import EdgeDir
from jaclang.pycore.constructs import Archetype as Obj
from jaclang.pycore.constructs import EdgeArchetype as Edge
from jaclang.pycore.constructs import NodeArchetype as Node
from jaclang.pycore.constructs import WalkerArchetype as Walker
from jaclang.pycore.runtime import JacRuntimeInterface

__all__ = [
    # Archetypes
    "Node",
    "Edge",
    "Walker",
    "Obj",
    "Root",
    "GenericEdge",
    "OPath",
    "DSFunc",
    # Constants
    "EdgeDir",
    # Core runtime functions
    "root",
    "spawn",
    "visit",
    "disengage",
    "connect",
    "disconnect",
    "create_j_context",
    "get_context",
    # Code generation helpers
    "impl_patch_filename",
    "jac_test",
    "jsx",
    "field",
    "log_report",
    "log_report_yield",
    "refs",
    "arefs",
    "filter_on",
    "assign_all",
    "safe_subscript",
    "build_edge",
    "destroy",
    "get_mtir",
    "sem",
    "call_llm",
    "by_operator",
    "on_entry",
    "on_exit",
    "thread_run",
    "thread_wait",
]

# Direct function references from JacRuntimeInterface (inherited by JacRuntime)
# Core runtime functions
connect = JacRuntimeInterface.connect
create_j_context = JacRuntimeInterface.create_j_context
disconnect = JacRuntimeInterface.disconnect
disengage = JacRuntimeInterface.disengage
get_context = JacRuntimeInterface.get_context
root = JacRuntimeInterface.root
spawn = JacRuntimeInterface.spawn
visit = JacRuntimeInterface.visit

# Code generation helpers
arefs = JacRuntimeInterface.arefs
assign_all = JacRuntimeInterface.assign_all
build_edge = JacRuntimeInterface.build_edge
by_operator = JacRuntimeInterface.by_operator
call_llm = JacRuntimeInterface.call_llm
destroy = JacRuntimeInterface.destroy
field = JacRuntimeInterface.field
filter_on = JacRuntimeInterface.filter_on
get_mtir = JacRuntimeInterface.get_mtir
impl_patch_filename = JacRuntimeInterface.impl_patch_filename
jac_test = JacRuntimeInterface.jac_test
jsx = JacRuntimeInterface.jsx
log_report = JacRuntimeInterface.log_report
log_report_yield = JacRuntimeInterface.log_report_yield
refs = JacRuntimeInterface.refs
safe_subscript = JacRuntimeInterface.safe_subscript
sem = JacRuntimeInterface.sem
on_entry = JacRuntimeInterface.on_entry
on_exit = JacRuntimeInterface.on_exit
thread_run = JacRuntimeInterface.thread_run
thread_wait = JacRuntimeInterface.thread_wait



from typing import Any
from typing_extensions import Self

class ExceptionCauseMixin(Exception):
    cause: Any
    def __new__(cls, *args, **kw) -> Self: ...
    def get_str(self) -> str: ...

class MathError(ExceptionCauseMixin, ValueError): ...

__all__ = ["ExceptionCauseMixin"]



from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Generator, Iterable
from typing import Final, Literal, TypeVar, overload

_T = TypeVar("_T")

STDOUT: Final = 1
STDERR: Final = 2

class SocketError(Exception): ...

NPIPE_ENDED: Final = 109

def read(socket, n: int = 4096): ...
def read_exactly(socket, n: int) -> bytes: ...
def next_frame_header(socket) -> tuple[Incomplete, int]: ...
def frames_iter(socket, tty): ...
def frames_iter_no_tty(socket) -> Generator[tuple[str | Incomplete, str | bytes | Incomplete]]: ...
def frames_iter_tty(socket) -> Generator[Incomplete]: ...
@overload
def consume_socket_output(
    frames: Iterable[tuple[Incomplete, Incomplete]], demux: Literal[True]
) -> tuple[Incomplete, Incomplete]: ...
@overload
def consume_socket_output(frames: Iterable[ReadableBuffer], demux: Literal[False] = False) -> bytes: ...
@overload
def demux_adaptor(stream_id: Literal[1], data: _T) -> tuple[_T, None]: ...
@overload
def demux_adaptor(stream_id: Literal[2], data: _T) -> tuple[None, _T]: ...



"""Tree Printing Helpers for Jac."""

from __future__ import annotations

import ast as ast3
import builtins
import html
from typing import TYPE_CHECKING

import jaclang.pycore.unitree as uni

if TYPE_CHECKING:
    from jaclang.pycore.unitree import UniNode, UniScopeNode

id_bag: dict = {}
id_used: int = 0
CLASS_COLOR_MAP: dict[str, str] = {
    "Expr": "#fbeb93",
    "AtomExpr": "#fbeb93",
    "ElementStmt": "#d8f3dc",
    "ArchBlockStmt": "#a8dadc",
    "EnumBlockStmt": "#89c6b3",
    "CodeBlockStmt": "#ffc596",
    "AstImplOnlyNode": "#a8dadc",
    "AstImplNeedingNode": "#a3d3f0",
    "NameAtom": "#cafafa",
    "ArchSpec": "#a3d3f0",
    "MatchPattern": "#a0d8b3",
    "SubTag": "#cafafa",
    "Module": "#D1FFE2  ",
    "GlobalVars": "#e1d1ff",
    "Test": "#e1d1ff",
    "ModuleCode": "#e1d1ff",
    "PyInlineCode": "#e1d1ff",
    "Import": "#e1d1ff",
    "ModulePath": "#e1d1ff",
    "ModuleItem": "#e1d1ff",
    "Archetype": "#a8dadc",
    "ArchDef": "#a3d3f0",
    "Enum": "#89c6b3",
    "EnumDef": "#89c6b3",
    "Ability": "#f9d69e",
    "AbilityDef": "#f9d69e",
    "FuncSignature": "#f9d69e",
    "EventSignature": "#f9d69e",
    "ArchRefChain": "#a3d3f0",
    "ParamVar": "#ffc596",
    "ArchHas": "#a8dadc",
    "HasVar": "#a3d3f0",
    "TypedCtxBlock": "#ffc596",
    "IfStmt": "#ccee9b",
    "ElseIf": "#ccee9b",
    "ElseStmt": "#ccee9b",
    "ExprStmt": "#d8f3dc",
    "TryStmt": "#ccee9b",
    "Except": "#d8f3dc",
    "FinallyStmt": "#d8f3dc",
    "IterForStmt": "#ccee9b",
    "InForStmt": "#ccee9b",
    "WhileStmt": "#ccee9b",
    "WithStmt": "#ccee9b",
    "ExprAsItem": "#ffc596",
    "RaiseStmt": "#d8f3dc",
    "AssertStmt": "#d8f3dc",
    "CheckStmt": "#d8f3dc",
    "CtrlStmt": "#d8f3dc",
    "DeleteStmt": "#d8f3dc",
    "ReportStmt": "#d8f3dc",
    "ReturnStmt": "#d8f3dc",
    "IgnoreStmt": "#d8f3dc",
    "VisitStmt": "#d8f3dc",
    "RevisitStmt": "#d8f3dc",
    "DisengageStmt": "#d8f3dc",
    "AwaitExpr": "#fbeb93",
    "GlobalStmt": "#ffc596",
    "NonLocalStmt": "#ffc596",
    "Assignment": "#ffc596",
    "BinaryExpr": "#fbeb93",
    "CompareExpr": "#fbeb93",
    "BoolExpr": "#fbeb93",
    "LambdaExpr": "#fbeb93",
    "UnaryExpr": "#fbeb93",
    "IfElseExpr": "#fbeb93",
    "MultiString": "#fccca4",
    "FString": "#ffc596",
    "ListVal": "#ffc596",
    "SetVal": "#ffc596",
    "TupleVal": "#ffc596",
    "DictVal": "#ffc596",
    "KVPair": "#ffc596",
    "KWPair": "#ffc596",
    "InnerCompr": "#fdc4df",
    "ListCompr": "#fdc4df",
    "GenCompr": "#ffc1de",
    "SetCompr": "#fdc4df",
    "DictCompr": "#fdc4df",
    "AtomTrailer": "#ffc596",
    "AtomUnit": "#ffc596",
    "YieldExpr": "#fbeb93",
    "FuncCall": "#fbeb93",
    "IndexSlice": "#fbeb93",
    "ArchRef": "#a8dadc",
    "EdgeRefTrailer": "#fbeb93",
    "EdgeOpRef": "#fbeb93",
    "DisconnectOp": "#fbeb93",
    "ConnectOp": "#fbeb93",
    "FilterCompr": "#fdc4df",
    "AssignCompr": "#fdc4df",
    "MatchStmt": "#a0d8b3",
    "MatchCase": "#a0d8b3",
    "MatchOr": "#a0d8b3",
    "MatchAs": "#a0d8b3",
    "MatchWild": "#a0d8b3",
    "MatchValue": "#a0d8b3",
    "MatchSingleton": "#a0d8b3",
    "MatchSequence": "#a0d8b3",
    "MatchMapping": "#a0d8b3",
    "MatchKVPair": "#a0d8b3",
    "MatchStar": "#a0d8b3",
    "MatchArch": "#a0d8b3",
    "Token": "#f0efef",
    "Name": "#cafafa",
    "SpecialVarRef": "#cafafa",
    "Literal": "#f1cfc4",
    "BuiltinType": "#f1cfc4",
    "Float": "#f1cfc4",
    "Int": "#f1cfc4",
    "String": "#f1cfc4",
    "Bool": "#f1cfc4",
    "Null": "#f1cfc4",
    "Ellipsis": "#f1cfc4",
    "EmptyToken": "#f1cfc4",
    "Semi": "#f1cfc4",
}


def printgraph_ast_tree(
    root: UniNode,
    dot_lines: list[str] | None = None,
) -> str:
    """Recursively generate ast tree in dot format."""
    starting_call = False
    if dot_lines is None:
        starting_call = True
        dot_lines = []

    def gen_node_id(node: uni.UniNode) -> int:
        """Generate number for each nodes."""
        global id_used
        if id(node) not in id_bag:
            id_bag[id(node)] = id_used
            id_used += 1
        return id_bag[id(node)]

    def gen_node_parameters(node: uni.UniNode) -> str:
        shape = ""
        fillcolor = ""
        style = ""
        _class__ = str(node.__class__)[33:-2]
        if _class__ in CLASS_COLOR_MAP:
            shape = 'shape="oval"'
            style = 'style="filled"'
            fillcolor = f'fillcolor="{CLASS_COLOR_MAP[_class__]}"'
        info1: list[tuple[str, str, str]] = []
        if isinstance(node, uni.Token):
            """ "Only tokens and some declared types are box(others are oval )"""
            shape = 'shape="box"'
            info1.append(("name", "=", node.name))
            info1.append(("value", "=", html.escape(node.value)))

        if len(info1) == 0:
            label = f'"{node.__class__.__name__}"'
        else:
            label = f"<{node.__class__.__name__}"
            for i in info1:
                label += f"<BR/> {i[0]}{i[1]}{i[2]}"
            label += ">"

        label = f"{label} {shape} {style} {fillcolor}".strip()
        return f"[label={label}]"

    dot_lines.append(f"{gen_node_id(root)} {gen_node_parameters(root)};")
    for i in root.kid:
        dot_lines.append(f"{gen_node_id(root)}  -> {gen_node_id(i)};")
        printgraph_ast_tree(i, dot_lines)
    if starting_call:
        return "\ndigraph graph1 {\n" + "\n".join(list(set(dot_lines))) + "\n}"
    return " "


def print_ast_tree(
    root: UniNode | ast3.AST,
    marker: str = "+-- ",
    level_markers: list[bool] | None = None,
    output_file: str | None = None,
    max_depth: int | None = None,
) -> str:
    """Recursively print ast tree."""
    from jaclang.pycore.unitree import AstSymbolNode, Token

    print_py_raise: bool = False

    def __node_repr_in_tree(node: UniNode) -> str:
        access = (
            f"Access: {node.access.tag.value} ,"
            if isinstance(node, uni.AstAccessNode) and node.access is not None
            else ""
        )
        sym_table_link = (
            f"SymbolTable: {node.type_sym_tab.scope_name}"
            if isinstance(node, AstSymbolNode) and node.type_sym_tab
            else "SymbolTable: None"
            if isinstance(node, AstSymbolNode)
            else ""
        )

        if isinstance(node, Token) and isinstance(node, AstSymbolNode):
            out = f"{node.__class__.__name__} - {node.value} - Type: {node.expr_type}, {access} {sym_table_link}"
            return out
        elif isinstance(node, Token):
            return f"{node.__class__.__name__} - {node.value}, {access}"
        elif (
            isinstance(node, uni.Module)
            and node.is_raised_from_py
            and not print_py_raise
        ):
            return f"{node.__class__.__name__} - PythonModuleRaised: {node.name}"
        elif isinstance(node, uni.ModuleItem):
            name_str = (
                node.name.sym_name
                if isinstance(node.name, uni.Name)
                else node.name.value
            )
            out = f"{node.__class__.__name__} - {name_str} - abs_path: {node.abs_path}"
            return out
        elif isinstance(node, uni.ModulePath):
            out = (
                f"{node.__class__.__name__} - {'.'.join([n.value for n in node.path] if node.path else [])} - "
                f"abs_path: {node.abs_path}"
            )
            return out
        elif isinstance(node, AstSymbolNode):
            out = f"{node.__class__.__name__} - {node.sym_name} - Type: {node.expr_type}, {access} {sym_table_link}"
            return out
        elif isinstance(node, uni.Expr):
            return f"{node.__class__.__name__} - Type: {node.expr_type}"
        else:
            return f"{node.__class__.__name__}, {access}"

    def __node_repr_in_py_tree(node: ast3.AST) -> str:
        if isinstance(node, ast3.Constant):
            return f"{node.__class__.__name__} - {node.value!r}"
        elif isinstance(node, ast3.Name):
            return f"{node.__class__.__name__} - {node.id}"
        elif isinstance(node, ast3.FunctionDef | ast3.ClassDef | ast3.AsyncFunctionDef):
            return f"{node.__class__.__name__} - {node.name}"
        elif isinstance(node, ast3.Import):
            return f"{node.__class__.__name__} - {', '.join(alias.name for alias in node.names)}"
        elif isinstance(node, ast3.ImportFrom):
            return f"{node.__class__.__name__} - {node.module} : {', '.join(alias.name for alias in node.names)}"
        elif isinstance(node, ast3.alias):
            return f"{node.__class__.__name__} - {node.name}"
        elif isinstance(node, ast3.Attribute):
            return f"{node.__class__.__name__} - {node.attr}"
        elif isinstance(node, ast3.Call):
            if isinstance(node.func, ast3.Name):
                return f"{node.__class__.__name__} - {node.func.id}"
            elif isinstance(node.func, ast3.Attribute):
                return f"{node.__class__.__name__} - {node.func.attr}"
            else:
                return f"{node.__class__.__name__}"
        else:
            return f"{node.__class__.__name__}"

    def get_location_info(node: ast3.AST) -> str:
        if (
            hasattr(node, "lineno")
            and hasattr(node, "col_offset")
            and hasattr(node, "end_lineno")
            and hasattr(node, "end_col_offset")
        ):
            start_pos = f"{node.lineno}:{node.col_offset + 1}"
            end_pos = f"{node.end_lineno}:{node.end_col_offset + 1 if node.end_col_offset else node.col_offset}"
            prefix = f"{start_pos} - {end_pos}"
            return prefix
        return "-:- - -:-"

    if root is None or (
        max_depth is not None and len(level_markers or []) >= max_depth
    ):
        return ""

    empty_str = " " * len(marker)
    connection_str = "|" + empty_str[:-1]
    if not level_markers:
        level_markers = []
    level = len(level_markers)  # recursion level

    def mapper(draw: bool) -> str:
        return connection_str if draw else empty_str

    markers = "".join(map(mapper, level_markers[:-1]))
    markers += marker if level > 0 else ""

    if isinstance(root, uni.UniNode):
        tree_str = f"{root.loc}\t{markers}{__node_repr_in_tree(root)}\n"
        if (
            isinstance(root, uni.Module)
            and root.is_raised_from_py
            and not print_py_raise
        ):
            kids: list[UniNode] = [
                *filter(
                    lambda x: x.is_raised_from_py,
                    root.get_all_sub_nodes(uni.Module),
                )
            ]
        else:
            kids = root.kid
        for i, child in enumerate(kids):
            is_last = i == len(kids) - 1
            tree_str += print_ast_tree(
                child, marker, [*level_markers, not is_last], output_file, max_depth
            )

    elif isinstance(root, ast3.AST):
        tree_str = (
            f"{get_location_info(root)}\t{markers}{__node_repr_in_py_tree(root)}\n"
        )
        for i_a, child_a in enumerate(ast3.iter_child_nodes(root)):
            is_last = i_a == len(list(ast3.iter_child_nodes(root))) - 1
            tree_str += print_ast_tree(
                child_a, marker, [*level_markers, not is_last], output_file, max_depth
            )

    # Write to file only at the top level call
    if output_file and level == 0:
        with open(output_file, "w") as f:
            f.write(tree_str)

    return tree_str


class SymbolTree:
    """Symbol Tree Node."""

    def __init__(
        self,
        node_name: str,
        parent: SymbolTree | None = None,
        children: list[SymbolTree] | None = None,
    ) -> None:
        """Initialize Symbol Tree Node."""
        self.parent = parent
        self.kid = children if children is not None else []
        self.name = node_name

    @property
    def parent(self) -> SymbolTree | None:
        """Get parent node."""
        return self.__parent

    @parent.setter
    def parent(self, parent_node: SymbolTree | None) -> None:
        """Set parent node."""
        if parent_node:
            self.__parent = parent_node
            parent_node.kid.append(self)


def _build_symbol_tree_common(
    node: UniScopeNode, parent_node: SymbolTree | None = None
) -> SymbolTree:
    root = SymbolTree(
        node_name=f"SymTable::{node.__class__.__name__}({node.scope_name})",
        parent=parent_node,
    )
    symbols = SymbolTree(node_name="Symbols", parent=root)
    children = SymbolTree(node_name="Sub Tables", parent=root)

    syms_to_iterate = set(node.names_in_scope.values())

    for sym in syms_to_iterate:
        symbol_node = SymbolTree(node_name=f"{sym.sym_name}", parent=symbols)
        SymbolTree(node_name=f"{sym.access} {sym.sym_type}", parent=symbol_node)

        if sym.decl and sym.decl.loc.first_line > 0:
            SymbolTree(
                node_name=f"decl: line {sym.decl.loc.first_line}, col {sym.decl.loc.col_start}",
                parent=symbol_node,
            )
            defn = SymbolTree(node_name="defn", parent=symbol_node)
            [
                SymbolTree(
                    node_name=f"line {n.loc.first_line}, col {n.loc.col_start}",
                    parent=defn,
                )
                for n in sym.defn
            ]
            uses = SymbolTree(node_name="uses", parent=symbol_node)
            [
                SymbolTree(
                    node_name=f"line {n.loc.first_line}, col {n.loc.col_start}",
                    parent=uses,
                )
                for n in sym.uses
            ]

    for k in node.kid_scope:
        if k.scope_name == "builtins":
            continue
        _build_symbol_tree_common(k, children)

    return root


def print_symtab_tree(
    root: UniScopeNode,
    marker: str = "+-- ",
    level_markers: list[bool] | None = None,
    output_file: str | None = None,
    depth: int | None = None,
) -> str:
    """Recursively print symbol table tree."""
    return get_symtab_tree_str(
        _build_symbol_tree_common(root),
        marker,
        level_markers,
        output_file,
        depth,
    )


def get_symtab_tree_str(
    root: SymbolTree,
    marker: str = "+-- ",
    level_markers: list[bool] | None = None,
    output_file: str | None = None,
    depth: int | None = None,
) -> str:
    """Recursively print symbol table tree."""
    if root is None or depth == 0 or root.name in dir(builtins):
        return ""

    level_markers = level_markers or []
    markers = "".join(
        [
            "|" + " " * (len(marker) - 1) if draw else " " * len(marker)
            for draw in level_markers[:-1]
        ]
    ) + (marker if level_markers else "")
    line = f"{markers}{root.name}\n"

    if output_file:
        with open(output_file, "a+") as f:
            f.write(line)

    return line + "".join(
        get_symtab_tree_str(
            child,
            marker,
            level_markers + [i < len(root.kid) - 1],
            output_file,
            None if depth is None else depth - 1,
        )
        for i, child in enumerate(root.kid)
    )


def printgraph_symtab_tree(node: UniScopeNode) -> str:
    """Generate DOT graph representation of a symbol table tree."""
    dot_lines = []
    id_map = {}
    last_id_used = 0

    def gen_node_id(node: SymbolTree) -> int:
        nonlocal last_id_used
        if id(node) not in id_map:
            id_map[id(node)] = last_id_used
            last_id_used += 1
        return id_map[id(node)]

    def gen_node_parameters(node: SymbolTree) -> str:
        shape = ""
        fillcolor = ""
        style = ""
        label = f'"{node.name}"'
        label = f"{label} {shape} {style} {fillcolor}".strip()
        return f"[label={label}]"

    def gen_dot_graph(node: SymbolTree) -> None:
        dot_lines.append(f"{gen_node_id(node)} {gen_node_parameters(node)};")
        for kid_node in node.kid:
            if kid_node:
                if kid_node.name in dir(builtins):
                    continue
                dot_lines.append(f"{gen_node_id(node)}  -> {gen_node_id(kid_node)};")
                gen_dot_graph(kid_node)

    gen_dot_graph(_build_symbol_tree_common(node))

    dot_str = "digraph graph1 {\n" + "\n".join(dot_lines) + "\n}"

    return dot_str



from _typeshed import Incomplete
from collections import deque
from collections.abc import Callable, Iterator
from typing import TypedDict, overload, type_check_only

from ._inputstream import HTMLBinaryInputStream, HTMLUnicodeInputStream, _BinaryInputStream, _UnicodeInputStream
from ._trie import Trie

@type_check_only
class _DataVars(TypedDict, total=False):
    data: str | None
    charAsInt: int

@type_check_only
class _Token(TypedDict, total=False):
    type: int
    data: str | list[str]
    datavars: _DataVars
    name: str
    selfClosing: bool
    selfClosingAcknowledged: bool
    publicId: str | None
    systemId: str | None
    correct: bool

entitiesTrie: Trie
attributeMap = dict

class HTMLTokenizer:
    # TODO: Use Protocol to allow subclasses to set `stream` that do not inherit from HTMLUnicodeInputStream
    stream: HTMLUnicodeInputStream | HTMLBinaryInputStream
    parser: Incomplete
    escapeFlag: bool
    lastFourChars: list[Incomplete]
    state: Callable[[], bool]
    escape: bool
    currentToken: _Token | None
    @overload
    def __init__(self, stream: _UnicodeInputStream, parser=None) -> None: ...
    @overload
    def __init__(
        self,
        stream: _BinaryInputStream,
        parser=None,
        *,
        override_encoding: str | bytes | None = None,
        transport_encoding: str | bytes | None = None,
        same_origin_parent_encoding: str | bytes | None = None,
        likely_encoding: str | bytes | None = None,
        default_encoding: str = "windows-1252",
        useChardet: bool = True,
    ) -> None: ...
    tokenQueue: deque[_Token]
    def __iter__(self) -> Iterator[_Token]: ...
    def consumeNumberEntity(self, isHex: bool | None) -> str: ...
    def consumeEntity(self, allowedChar: str | None = None, fromAttribute: bool = False) -> None: ...
    def processEntityInAttribute(self, allowedChar: str | None) -> None: ...
    def emitCurrentToken(self) -> None: ...
    def dataState(self) -> bool: ...
    def entityDataState(self) -> bool: ...
    def rcdataState(self) -> bool: ...
    def characterReferenceInRcdata(self) -> bool: ...
    def rawtextState(self) -> bool: ...
    def scriptDataState(self) -> bool: ...
    def plaintextState(self) -> bool: ...
    def tagOpenState(self) -> bool: ...
    def closeTagOpenState(self) -> bool: ...
    def tagNameState(self) -> bool: ...
    temporaryBuffer: str
    def rcdataLessThanSignState(self) -> bool: ...
    def rcdataEndTagOpenState(self) -> bool: ...
    def rcdataEndTagNameState(self) -> bool: ...
    def rawtextLessThanSignState(self) -> bool: ...
    def rawtextEndTagOpenState(self) -> bool: ...
    def rawtextEndTagNameState(self) -> bool: ...
    def scriptDataLessThanSignState(self) -> bool: ...
    def scriptDataEndTagOpenState(self) -> bool: ...
    def scriptDataEndTagNameState(self) -> bool: ...
    def scriptDataEscapeStartState(self) -> bool: ...
    def scriptDataEscapeStartDashState(self) -> bool: ...
    def scriptDataEscapedState(self) -> bool: ...
    def scriptDataEscapedDashState(self) -> bool: ...
    def scriptDataEscapedDashDashState(self) -> bool: ...
    def scriptDataEscapedLessThanSignState(self) -> bool: ...
    def scriptDataEscapedEndTagOpenState(self) -> bool: ...
    def scriptDataEscapedEndTagNameState(self) -> bool: ...
    def scriptDataDoubleEscapeStartState(self) -> bool: ...
    def scriptDataDoubleEscapedState(self) -> bool: ...
    def scriptDataDoubleEscapedDashState(self) -> bool: ...
    def scriptDataDoubleEscapedDashDashState(self) -> bool: ...
    def scriptDataDoubleEscapedLessThanSignState(self) -> bool: ...
    def scriptDataDoubleEscapeEndState(self) -> bool: ...
    def beforeAttributeNameState(self) -> bool: ...
    def attributeNameState(self) -> bool: ...
    def afterAttributeNameState(self) -> bool: ...
    def beforeAttributeValueState(self) -> bool: ...
    def attributeValueDoubleQuotedState(self) -> bool: ...
    def attributeValueSingleQuotedState(self) -> bool: ...
    def attributeValueUnQuotedState(self) -> bool: ...
    def afterAttributeValueState(self) -> bool: ...
    def selfClosingStartTagState(self) -> bool: ...
    def bogusCommentState(self) -> bool: ...
    def markupDeclarationOpenState(self) -> bool: ...
    def commentStartState(self) -> bool: ...
    def commentStartDashState(self) -> bool: ...
    def commentState(self) -> bool: ...
    def commentEndDashState(self) -> bool: ...
    def commentEndState(self) -> bool: ...
    def commentEndBangState(self) -> bool: ...
    def doctypeState(self) -> bool: ...
    def beforeDoctypeNameState(self) -> bool: ...
    def doctypeNameState(self) -> bool: ...
    def afterDoctypeNameState(self) -> bool: ...
    def afterDoctypePublicKeywordState(self) -> bool: ...
    def beforeDoctypePublicIdentifierState(self) -> bool: ...
    def doctypePublicIdentifierDoubleQuotedState(self) -> bool: ...
    def doctypePublicIdentifierSingleQuotedState(self) -> bool: ...
    def afterDoctypePublicIdentifierState(self) -> bool: ...
    def betweenDoctypePublicAndSystemIdentifiersState(self) -> bool: ...
    def afterDoctypeSystemKeywordState(self) -> bool: ...
    def beforeDoctypeSystemIdentifierState(self) -> bool: ...
    def doctypeSystemIdentifierDoubleQuotedState(self) -> bool: ...
    def doctypeSystemIdentifierSingleQuotedState(self) -> bool: ...
    def afterDoctypeSystemIdentifierState(self) -> bool: ...
    def bogusDoctypeState(self) -> bool: ...
    def cdataSectionState(self) -> bool: ...



class md4:
    name: str
    digest_size: int
    digestsize: int
    block_size: int
    def __init__(self, content=None) -> None: ...
    def update(self, content) -> None: ...
    def copy(self): ...
    def digest(self): ...
    def hexdigest(self): ...

__all__ = ["md4"]



import argparse
from _typeshed import ConvertibleToInt
from collections.abc import Callable, Container
from ssl import SSLContext, _SSLMethod
from typing import Annotated, Any, ClassVar, overload
from typing_extensions import TypeAlias

from gunicorn.arbiter import Arbiter
from gunicorn.glogging import Logger as GLogger
from gunicorn.http import Request
from gunicorn.http.wsgi import Response
from gunicorn.workers.base import Worker

from ._types import _AddressType, _EnvironType

_ConfigValueType: TypeAlias = Any
# Any: Value type depends on the setting's validator (e.g., validate_bool, validate_pos_int).
# Maybe types: bool, int, str, list[str], dict[str, Any], Callable[..., object], type[object], ssl.PROTOCOL_*.
# Validator ensures type correctness at runtime.

_OnStartingHookType: TypeAlias = Callable[[Arbiter], object]
_OnReloadHookType: TypeAlias = Callable[[Arbiter], object]
_WhenReadyHookType: TypeAlias = Callable[[Arbiter], object]
_PreForkHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_PostForkHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_PostWorkerInitHookType: TypeAlias = Callable[[Worker], object]
_WorkerIntHookType: TypeAlias = Callable[[Worker], object]
_WorkerAbortHookType: TypeAlias = Callable[[Worker], object]
_PreExecHookType: TypeAlias = Callable[[Arbiter], object]
_PreRequestHookType: TypeAlias = Callable[[Worker, Request], object]
_PostRequestHookType: TypeAlias = Callable[[Worker, Request, _EnvironType, Response], object]
_ChildExitHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_WorkerExitHookType: TypeAlias = Callable[[Arbiter, Worker], object]
_NumWorkersChangedHookType: TypeAlias = Callable[[Arbiter, int, int | None], object]
_OnExitHookType: TypeAlias = Callable[[Arbiter], object]
_SSLContextHookType: TypeAlias = Callable[[Config, Callable[[], SSLContext]], SSLContext]

_HookType: TypeAlias = (
    _OnStartingHookType
    | _OnReloadHookType
    | _WhenReadyHookType
    | _PreForkHookType
    | _PostForkHookType
    | _PostWorkerInitHookType
    | _WorkerIntHookType
    | _WorkerAbortHookType
    | _PreExecHookType
    | _PreRequestHookType
    | _PostRequestHookType
    | _ChildExitHookType
    | _WorkerExitHookType
    | _NumWorkersChangedHookType
    | _OnExitHookType
    | _SSLContextHookType
)
# Validators
_BoolValidatorType: TypeAlias = Callable[[bool | str | None], bool | None]
_StringValidatorType: TypeAlias = Callable[[str | None], str | None]
_ListStringValidatorType: TypeAlias = Callable[[str | list[str] | None], list[str]]
_IntValidatorType: TypeAlias = Callable[[int | ConvertibleToInt], int]
_DictValidatorType: TypeAlias = Callable[[dict[str, Any]], dict[str, Any]]
_ClassValidatorType: TypeAlias = Callable[[object | str | None], type[Any] | None]
_UserGroupValidatorType: TypeAlias = Callable[[str | int | None], int]
_AddressValidatorType: TypeAlias = Callable[[str | None], _AddressType | None]
_CallableValidatorType: TypeAlias = Callable[[str | _HookType], _HookType]

_ValidatorType: TypeAlias = (  # noqa: Y047
    _BoolValidatorType
    | _StringValidatorType
    | _ListStringValidatorType
    | _IntValidatorType
    | _DictValidatorType
    | _ClassValidatorType
    | _UserGroupValidatorType
    | _AddressValidatorType
    | _CallableValidatorType
)

KNOWN_SETTINGS: list[Setting]
PLATFORM: str

def make_settings(ignore: Container[Setting] | None = None) -> dict[str, Setting]: ...
def auto_int(_: Any, x: str) -> int: ...

class Config:
    settings: dict[str, Setting]
    usage: str | None
    prog: str | None
    env_orig: dict[str, str]

    def __init__(self, usage: str | None = None, prog: str | None = None) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def set(self, name: str, value: _ConfigValueType) -> None: ...
    def get_cmd_args_from_env(self) -> list[str]: ...
    def parser(self) -> argparse.ArgumentParser: ...
    @property
    def worker_class_str(self) -> str: ...
    @property
    def worker_class(self) -> type[Worker]: ...
    @property
    def address(self) -> list[_AddressType]: ...
    @property
    def uid(self) -> int: ...
    @property
    def gid(self) -> int: ...
    @property
    def proc_name(self) -> str | None: ...
    @property
    def logger_class(self) -> type[GLogger]: ...
    @property
    def is_ssl(self) -> bool: ...
    @property
    def ssl_options(self) -> dict[str, Any]: ...
    @property
    def env(self) -> dict[str, str]: ...
    @property
    def sendfile(self) -> bool: ...
    @property
    def reuse_port(self) -> bool: ...
    @property
    def paste_global_conf(self) -> dict[str, str] | None: ...

class SettingMeta(type):
    def __new__(cls, name: str, bases: tuple[type, ...], attrs: dict[str, Any]) -> SettingMeta: ...
    def fmt_desc(cls, desc: str) -> None: ...

class Setting(metaclass=SettingMeta):
    name: ClassVar[str | None]
    value: _ConfigValueType
    section: ClassVar[str | None]
    cli: ClassVar[list[str] | None]
    validator: ClassVar[Callable[..., Any] | None]  # See `_ValidatorType`
    type: ClassVar[argparse._ActionType | None]
    meta: ClassVar[str | None]
    action: ClassVar[str | None]
    default: ClassVar[Any]
    short: ClassVar[str | None]
    desc: ClassVar[str | None]
    nargs: ClassVar[int | str | None]
    const: ClassVar[bool | None]
    order: ClassVar[int]

    def __init__(self) -> None: ...
    def add_option(self, parser: argparse.ArgumentParser) -> None: ...
    def copy(self) -> Setting: ...
    def get(self) -> _ConfigValueType: ...
    def set(self, val: _ConfigValueType) -> None: ...
    def __lt__(self, other: Setting) -> bool: ...

    __cmp__ = __lt__

@overload
def validate_bool(val: bool) -> bool: ...
@overload
def validate_bool(val: None) -> None: ...
@overload
def validate_bool(val: Annotated[str, "Case-insensitive boolean string ('true'/'false' in any case)"]) -> bool: ...
def validate_dict(val: dict[str, Any]) -> dict[str, Any]: ...
@overload
def validate_pos_int(val: int) -> int: ...
@overload
def validate_pos_int(val: ConvertibleToInt) -> int: ...
def validate_ssl_version(val: _SSLMethod) -> _SSLMethod: ...
@overload
def validate_string(val: str) -> str: ...
@overload
def validate_string(val: None) -> None: ...
@overload
def validate_file_exists(val: str) -> str: ...
@overload
def validate_file_exists(val: None) -> None: ...
@overload
def validate_list_string(val: str) -> list[str]: ...
@overload
def validate_list_string(val: list[str]) -> list[str]: ...
@overload
def validate_list_string(val: None) -> list[str]: ...
@overload
def validate_list_of_existing_files(val: str) -> list[str]: ...
@overload
def validate_list_of_existing_files(val: list[str]) -> list[str]: ...
@overload
def validate_list_of_existing_files(val: None) -> list[str]: ...
def validate_string_to_addr_list(val: str | None) -> list[str]: ...
def validate_string_to_list(val: str | None) -> list[str]: ...
@overload
def validate_class(val: str) -> str: ...
@overload
def validate_class(val: None) -> None: ...
@overload
def validate_class(val: object) -> object: ...
def validate_callable(arity: int) -> _CallableValidatorType: ...
def validate_user(val: int | str | None) -> int: ...
def validate_group(val: int | str | None) -> int: ...
def validate_post_request(val: str | _HookType) -> _PostRequestHookType: ...
def validate_chdir(val: str) -> str: ...
@overload
def validate_statsd_address(val: str) -> _AddressType: ...
@overload
def validate_statsd_address(val: None) -> None: ...
def validate_reload_engine(val: str) -> str: ...
@overload
def validate_header_map_behaviour(val: str) -> str: ...
@overload
def validate_header_map_behaviour(val: None) -> None: ...
def get_default_config_file() -> str | None: ...

class ConfigFile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class WSGIApp(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class Bind(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class Backlog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Workers(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class WorkerClass(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ClassValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class WorkerThreads(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class WorkerConnections(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class MaxRequests(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class MaxRequestsJitter(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Timeout(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class GracefulTimeout(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Keepalive(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class LimitRequestLine(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class LimitRequestFields(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class LimitRequestFieldSize(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[type[int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Reload(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class ReloadEngine(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[Callable[[str], str]]
    default: ClassVar[str]
    desc: ClassVar[str]

class ReloadExtraFiles(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class Spew(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class ConfigCheck(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PrintConfig(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PreloadApp(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Sendfile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    const: ClassVar[bool]
    desc: ClassVar[str]

class ReusePort(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Chdir(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[Callable[[str], str]]
    default: ClassVar[str]
    default_doc: ClassVar[str]
    desc: ClassVar[str]

class Daemon(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Env(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class Pidfile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class WorkerTmpDir(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class User(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_UserGroupValidatorType]
    default: ClassVar[int]
    default_doc: ClassVar[str]
    desc: ClassVar[str]

class Group(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_UserGroupValidatorType]
    default: ClassVar[int]
    default_doc: ClassVar[str]
    desc: ClassVar[str]

class Umask(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_IntValidatorType]
    type: ClassVar[Callable[[Any, str], int]]
    default: ClassVar[int]
    desc: ClassVar[str]

class Initgroups(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class TmpUploadDir(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SecureSchemeHeader(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_DictValidatorType]
    default: ClassVar[dict[str, str]]
    desc: ClassVar[str]

class ForwardedAllowIPS(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class AccessLog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class DisableRedirectAccessToSyslog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class AccessLogFormat(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class ErrorLog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class Loglevel(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class CaptureOutput(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class LoggerClass(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ClassValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class LogConfig(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class LogConfigDict(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_DictValidatorType]
    default: ClassVar[dict[str, Any]]
    desc: ClassVar[str]

class LogConfigJson(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SyslogTo(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class Syslog(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class SyslogPrefix(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SyslogFacility(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class EnableStdioInheritance(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    default: ClassVar[bool]
    action: ClassVar[str]
    desc: ClassVar[str]

class StatsdHost(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    default: ClassVar[None]
    validator: ClassVar[_AddressValidatorType]
    desc: ClassVar[str]

class DogstatsdTags(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    default: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    desc: ClassVar[str]

class StatsdPrefix(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    default: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    desc: ClassVar[str]

class Procname(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class DefaultProcName(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class PythonPath(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class Paste(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class OnStarting(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType] = ...
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_OnStartingHookType]
    desc: ClassVar[str]

    def on_starting(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class OnReload(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_OnReloadHookType]
    desc: ClassVar[str]

    def on_reload(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WhenReady(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WhenReadyHookType]
    desc: ClassVar[str]

    def when_ready(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class Prefork(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PreForkHookType]
    desc: ClassVar[str]

    def pre_fork(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class Postfork(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PostForkHookType]
    desc: ClassVar[str]

    def post_fork(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PostWorkerInit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PostWorkerInitHookType]
    desc: ClassVar[str]

    def post_worker_init(worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WorkerInt(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WorkerIntHookType]
    desc: ClassVar[str]

    def worker_int(worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WorkerAbort(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WorkerAbortHookType]
    desc: ClassVar[str]

    def worker_abort(worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PreExec(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PreExecHookType]
    desc: ClassVar[str]

    def pre_exec(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PreRequest(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PreRequestHookType]
    desc: ClassVar[str]

    def pre_request(worker: Worker, req: Request) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class PostRequest(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_PostRequestHookType]
    desc: ClassVar[str]

    def post_request(worker: Worker, req: Request, environ: _EnvironType, resp: Response) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class ChildExit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_ChildExitHookType]
    desc: ClassVar[str]

    def child_exit(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class WorkerExit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_WorkerExitHookType]
    desc: ClassVar[str]

    def worker_exit(server: Arbiter, worker: Worker) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class NumWorkersChanged(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_NumWorkersChangedHookType]
    desc: ClassVar[str]

    def nworkers_changed(server: Arbiter, new_value: int, old_value: int | None) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class OnExit(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    default: ClassVar[_OnExitHookType]
    desc: ClassVar[str]

    def on_exit(server: Arbiter) -> None: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class NewSSLContext(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    validator: ClassVar[_CallableValidatorType]
    type: ClassVar[Callable[..., Any]]
    default: ClassVar[_SSLContextHookType]
    desc: ClassVar[str]

    def ssl_context(config: Config, default_ssl_context_factory: Callable[[], SSLContext]) -> SSLContext: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]

class ProxyProtocol(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    default: ClassVar[bool]
    action: ClassVar[str]
    desc: ClassVar[str]

class ProxyAllowFrom(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class KeyFile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class CertFile(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SSLVersion(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[Callable[[_SSLMethod], _SSLMethod]]
    default: ClassVar[_SSLMethod]
    desc: ClassVar[str]

class CertReqs(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_IntValidatorType]
    default: ClassVar[int]
    desc: ClassVar[str]

class CACerts(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class SuppressRaggedEOFs(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    action: ClassVar[str]
    default: ClassVar[bool]
    validator: ClassVar[_BoolValidatorType]
    desc: ClassVar[str]

class DoHandshakeOnConnect(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class Ciphers(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[None]
    desc: ClassVar[str]

class PasteGlobalConf(Setting):
    name: ClassVar[str]
    action: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    meta: ClassVar[str]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[list[str]]
    desc: ClassVar[str]

class PermitObsoleteFolding(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class StripHeaderSpaces(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PermitUnconventionalHTTPMethod(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class PermitUnconventionalHTTPVersion(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class CasefoldHTTPMethod(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_BoolValidatorType]
    action: ClassVar[str]
    default: ClassVar[bool]
    desc: ClassVar[str]

class ForwarderHeaders(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_ListStringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]

class HeaderMap(Setting):
    name: ClassVar[str]
    section: ClassVar[str]
    cli: ClassVar[list[str]]
    validator: ClassVar[_StringValidatorType]
    default: ClassVar[str]
    desc: ClassVar[str]



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["biadjacency_matrix", "from_biadjacency_matrix"]

@_dispatchable
def biadjacency_matrix(
    G: Graph[_Node],
    row_order: Iterable[_Node],
    column_order: Iterable[Incomplete] | None = None,
    dtype=None,
    weight: str | None = "weight",
    format="csr",
): ...  # Return is a complex union of scipy classes depending on the format param
@_dispatchable
def from_biadjacency_matrix(A, create_using: Graph[_Node] | None = None, edge_attribute: str = "weight"): ...



from collections.abc import Sequence

from .flowables import Flowable, _Container, _FindSplitterMixin

class MultiCol(_Container, _FindSplitterMixin, Flowable):
    contents: Sequence[Flowable]
    widths: Sequence[float | str]
    minHeightNeeded: float
    def __init__(
        self,
        contents: Sequence[Flowable],
        widths: Sequence[float | str],
        minHeightNeeded: float = 36,
        spaceBefore: float | None = None,
        spaceAfter: float | None = None,
    ) -> None: ...
    def nWidths(self, aW: float) -> list[float]: ...

__all__ = ["MultiCol"]



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["preflow_push"]

@_dispatchable
def preflow_push(
    G: Graph[_Node],
    s: _Node,
    t: _Node,
    capacity: str = "capacity",
    residual: Graph[_Node] | None = None,
    global_relabel_freq: float = 1,
    value_only: bool = False,
): ...



from logging import Logger
from re import Pattern

from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor

log: Logger
META_RE: Pattern[str]
META_MORE_RE: Pattern[str]
BEGIN_RE: Pattern[str]
END_RE: Pattern[str]

class MetaExtension(Extension):
    md: Markdown
    def reset(self) -> None: ...

class MetaPreprocessor(Preprocessor): ...

def makeExtension(**kwargs) -> MetaExtension: ...



from .parameters import prepare_revoke_token_request as prepare_revoke_token_request
from .revocation import RevocationEndpoint as RevocationEndpoint

__all__ = ["prepare_revoke_token_request", "RevocationEndpoint"]



from typing import Any, ClassVar
from typing_extensions import deprecated

import passlib.utils.handlers as uh
from passlib.ifc import DisabledHash

class unix_fallback(DisabledHash, uh.StaticHandler):
    name: ClassVar[str]
    @classmethod
    def identify(cls, hash: str | bytes) -> bool: ...
    enable_wildcard: Any
    def __init__(self, enable_wildcard: bool = False, **kwds) -> None: ...
    @classmethod
    def verify(cls, secret: str | bytes, hash: str | bytes, enable_wildcard: bool = False): ...  # type: ignore[override]

class unix_disabled(DisabledHash, uh.MinimalHandler):
    name: ClassVar[str]
    default_marker: ClassVar[str]
    setting_kwds: ClassVar[tuple[str, ...]]
    context_kwds: ClassVar[tuple[str, ...]]
    @classmethod
    def using(cls, marker=None, **kwds): ...  # type: ignore[override]
    @classmethod
    def identify(cls, hash: str | bytes) -> bool: ...
    @classmethod
    def verify(cls, secret: str | bytes, hash: str | bytes) -> bool: ...  # type: ignore[override]
    @classmethod
    def hash(cls, secret: str | bytes, **kwds) -> str: ...
    @classmethod
    def genhash(cls, secret: str | bytes, config, marker=None): ...  # type: ignore[override]
    @classmethod
    def disable(cls, hash: str | bytes | None = None) -> str: ...
    @classmethod
    def enable(cls, hash: str | bytes) -> str: ...

class plaintext(uh.MinimalHandler):
    name: ClassVar[str]
    default_encoding: ClassVar[str]
    setting_kwds: ClassVar[tuple[str, ...]]
    context_kwds: ClassVar[tuple[str, ...]]
    @classmethod
    def identify(cls, hash: str | bytes): ...
    @classmethod
    def hash(cls, secret: str | bytes, encoding=None): ...  # type: ignore[override]
    @classmethod
    def verify(cls, secret: str | bytes, hash: str | bytes, encoding: str | None = None): ...  # type: ignore[override]
    @deprecated("Deprecated since Passlib 1.7, will be removed in 2.0")
    @classmethod
    def genconfig(cls): ...  # type: ignore[override]
    @deprecated("Deprecated since Passlib 1.7, will be removed in 2.0")
    @classmethod
    def genhash(cls, secret, config, encoding: str | None = None): ...  # type: ignore[override]

__all__ = ["unix_disabled", "unix_fallback", "plaintext"]



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["subgraph_centrality_exp", "subgraph_centrality", "communicability_betweenness_centrality", "estrada_index"]

@_dispatchable
def subgraph_centrality_exp(G: Graph[_Node]) -> dict[Incomplete, float]: ...
@_dispatchable
def subgraph_centrality(G: Graph[_Node]) -> dict[Incomplete, float]: ...
@_dispatchable
def communicability_betweenness_centrality(G: Graph[_Node]) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def estrada_index(G: Graph[_Node]) -> float: ...



import datetime

from ..core import WesternCalendar

class NewZealand(WesternCalendar):
    def get_queens_birthday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_labour_day(self, year: int) -> tuple[datetime.date, str]: ...



from _typeshed import Incomplete

from antlr4 import DFA as DFA
from antlr4.atn.ATN import ATN as ATN
from antlr4.atn.ATNConfig import ATNConfig as ATNConfig
from antlr4.atn.ATNConfigSet import ATNConfigSet as ATNConfigSet
from antlr4.atn.ATNSimulator import ATNSimulator as ATNSimulator
from antlr4.atn.ATNState import ATNState as ATNState, DecisionState as DecisionState, RuleStopState as RuleStopState
from antlr4.atn.PredictionMode import PredictionMode as PredictionMode
from antlr4.atn.SemanticContext import SemanticContext as SemanticContext, andContext as andContext, orContext as orContext
from antlr4.atn.Transition import (
    ActionTransition as ActionTransition,
    AtomTransition as AtomTransition,
    NotSetTransition as NotSetTransition,
    PrecedencePredicateTransition as PrecedencePredicateTransition,
    PredicateTransition as PredicateTransition,
    RuleTransition as RuleTransition,
    SetTransition as SetTransition,
    Transition as Transition,
)
from antlr4.BufferedTokenStream import TokenStream as TokenStream
from antlr4.dfa.DFAState import DFAState as DFAState, PredPrediction as PredPrediction
from antlr4.error.Errors import NoViableAltException as NoViableAltException
from antlr4.Parser import Parser as Parser
from antlr4.ParserRuleContext import ParserRuleContext as ParserRuleContext
from antlr4.PredictionContext import (
    PredictionContext as PredictionContext,
    PredictionContextCache as PredictionContextCache,
    PredictionContextFromRuleContext as PredictionContextFromRuleContext,
    SingletonPredictionContext as SingletonPredictionContext,
)
from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token
from antlr4.Utils import str_list as str_list

class ParserATNSimulator(ATNSimulator):
    __slots__ = ("parser", "decisionToDFA", "predictionMode", "_input", "_startIndex", "_outerContext", "_dfa", "mergeCache")
    debug: bool
    trace_atn_sim: bool
    dfa_debug: bool
    retry_debug: bool
    parser: Incomplete
    decisionToDFA: Incomplete
    predictionMode: Incomplete
    mergeCache: Incomplete
    def __init__(
        self, parser: Parser, atn: ATN, decisionToDFA: list[DFA], sharedContextCache: PredictionContextCache
    ) -> None: ...
    def reset(self) -> None: ...
    def adaptivePredict(self, input: TokenStream, decision: int, outerContext: ParserRuleContext): ...
    def execATN(self, dfa: DFA, s0: DFAState, input: TokenStream, startIndex: int, outerContext: ParserRuleContext): ...
    def getExistingTargetState(self, previousD: DFAState, t: int): ...
    def computeTargetState(self, dfa: DFA, previousD: DFAState, t: int): ...
    def predicateDFAState(self, dfaState: DFAState, decisionState: DecisionState): ...
    def execATNWithFullContext(
        self, dfa: DFA, D: DFAState, s0: ATNConfigSet, input: TokenStream, startIndex: int, outerContext: ParserRuleContext
    ): ...
    def computeReachSet(self, closure: ATNConfigSet, t: int, fullCtx: bool): ...
    def removeAllConfigsNotInRuleStopState(self, configs: ATNConfigSet, lookToEndOfRule: bool): ...
    def computeStartState(self, p: ATNState, ctx: RuleContext, fullCtx: bool): ...
    def applyPrecedenceFilter(self, configs: ATNConfigSet): ...
    def getReachableTarget(self, trans: Transition, ttype: int): ...
    def getPredsForAmbigAlts(self, ambigAlts: set[int], configs: ATNConfigSet, nalts: int): ...
    def getPredicatePredictions(self, ambigAlts: set[int], altToPred: list[int]): ...
    def getSynValidOrSemInvalidAltThatFinishedDecisionEntryRule(self, configs: ATNConfigSet, outerContext: ParserRuleContext): ...
    def getAltThatFinishedDecisionEntryRule(self, configs: ATNConfigSet): ...
    def splitAccordingToSemanticValidity(self, configs: ATNConfigSet, outerContext: ParserRuleContext): ...
    def evalSemanticContext(self, predPredictions: list[Incomplete], outerContext: ParserRuleContext, complete: bool): ...
    def closure(
        self,
        config: ATNConfig,
        configs: ATNConfigSet,
        closureBusy: set[Incomplete],
        collectPredicates: bool,
        fullCtx: bool,
        treatEofAsEpsilon: bool,
    ): ...
    def closureCheckingStopState(
        self,
        config: ATNConfig,
        configs: ATNConfigSet,
        closureBusy: set[Incomplete],
        collectPredicates: bool,
        fullCtx: bool,
        depth: int,
        treatEofAsEpsilon: bool,
    ): ...
    def closure_(
        self,
        config: ATNConfig,
        configs: ATNConfigSet,
        closureBusy: set[Incomplete],
        collectPredicates: bool,
        fullCtx: bool,
        depth: int,
        treatEofAsEpsilon: bool,
    ): ...
    def canDropLoopEntryEdgeInLeftRecursiveRule(self, config): ...
    def getRuleName(self, index: int): ...
    epsilonTargetMethods: Incomplete
    def getEpsilonTarget(
        self, config: ATNConfig, t: Transition, collectPredicates: bool, inContext: bool, fullCtx: bool, treatEofAsEpsilon: bool
    ): ...
    def actionTransition(self, config: ATNConfig, t: ActionTransition): ...
    def precedenceTransition(
        self, config: ATNConfig, pt: PrecedencePredicateTransition, collectPredicates: bool, inContext: bool, fullCtx: bool
    ): ...
    def predTransition(
        self, config: ATNConfig, pt: PredicateTransition, collectPredicates: bool, inContext: bool, fullCtx: bool
    ): ...
    def ruleTransition(self, config: ATNConfig, t: RuleTransition): ...
    def getConflictingAlts(self, configs: ATNConfigSet): ...
    def getConflictingAltsOrUniqueAlt(self, configs: ATNConfigSet): ...
    def getTokenName(self, t: int): ...
    def getLookaheadName(self, input: TokenStream): ...
    def dumpDeadEndConfigs(self, nvae: NoViableAltException): ...
    def noViableAlt(self, input: TokenStream, outerContext: ParserRuleContext, configs: ATNConfigSet, startIndex: int): ...
    def getUniqueAlt(self, configs: ATNConfigSet): ...
    def addDFAEdge(self, dfa: DFA, from_: DFAState, t: int, to: DFAState): ...
    def addDFAState(self, dfa: DFA, D: DFAState): ...
    def reportAttemptingFullContext(
        self, dfa: DFA, conflictingAlts: set[Incomplete], configs: ATNConfigSet, startIndex: int, stopIndex: int
    ): ...
    def reportContextSensitivity(self, dfa: DFA, prediction: int, configs: ATNConfigSet, startIndex: int, stopIndex: int): ...
    def reportAmbiguity(
        self,
        dfa: DFA,
        D: DFAState,
        startIndex: int,
        stopIndex: int,
        exact: bool,
        ambigAlts: set[Incomplete],
        configs: ATNConfigSet,
    ): ...



from collections.abc import Callable, Iterable
from typing import TypeVar
from typing_extensions import TypeAlias

_FormatCheckCallable: TypeAlias = Callable[[object], bool]
_F = TypeVar("_F", bound=_FormatCheckCallable)
_RaisesType: TypeAlias = type[Exception] | tuple[type[Exception], ...]

class FormatChecker:
    checkers: dict[str, tuple[_FormatCheckCallable, _RaisesType]]

    def __init__(self, formats: Iterable[str] | None = None) -> None: ...
    def checks(self, format: str, raises: _RaisesType = ()) -> Callable[[_F], _F]: ...
    @classmethod
    def cls_checks(cls, format: str, raises: _RaisesType = ()) -> Callable[[_F], _F]: ...
    def check(self, instance: object, format: str) -> None: ...
    def conforms(self, instance: object, format: str) -> bool: ...

draft3_format_checker: FormatChecker
draft4_format_checker: FormatChecker
draft6_format_checker: FormatChecker
draft7_format_checker: FormatChecker
draft201909_format_checker: FormatChecker
draft202012_format_checker: FormatChecker

def is_email(instance: object) -> bool: ...
def is_ipv4(instance: object) -> bool: ...
def is_ipv6(instance: object) -> bool: ...

# is_host_name is only defined if fqdn is installed.
def is_host_name(instance: object) -> bool: ...
def is_idn_host_name(instance: object) -> bool: ...
def is_uri(instance: object) -> bool: ...
def is_uri_reference(instance: object) -> bool: ...
def is_iri(instance: object) -> bool: ...
def is_iri_reference(instance: object) -> bool: ...
def is_datetime(instance: object) -> bool: ...
def is_time(instance: object) -> bool: ...
def is_regex(instance: object) -> bool: ...
def is_date(instance: object) -> bool: ...
def is_draft3_time(instance: object) -> bool: ...
def is_css21_color(instance: object) -> bool: ...
def is_json_pointer(instance: object) -> bool: ...
def is_relative_json_pointer(instance: object) -> bool: ...
def is_uri_template(instance: object) -> bool: ...

# is_duration is only defined if isoduration is installed.
def is_duration(instance: object) -> bool: ...
def is_uuid(instance: object) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Iterator
from typing import ClassVar

class DataTableFormula:
    t: ClassVar[str]

    ref: Incomplete
    ca: bool
    dt2D: bool
    dtr: bool
    r1: Incomplete | None
    r2: Incomplete | None
    del1: bool
    del2: bool

    def __init__(
        self,
        ref,
        ca: bool = False,
        dt2D: bool = False,
        dtr: bool = False,
        r1=None,
        r2=None,
        del1: bool = False,
        del2: bool = False,
        **kw,
    ) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...

class ArrayFormula:
    t: ClassVar[str]
    ref: Incomplete
    text: Incomplete | None

    def __init__(self, ref, text=None) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...



from collections.abc import Container, Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_partition"]

@_dispatchable
def is_partition(G: Graph[_Node], communities: Iterable[Container[_Node]]) -> bool: ...



from collections.abc import Generator
from typing import Any

class ScryptEngine:
    n: int
    r: int
    p: int
    smix_bytes: int
    iv_bytes: int
    bmix_len: int
    bmix_half_len: int
    bmix_struct: Any
    integerify: Any
    @classmethod
    def execute(cls, secret, salt, n, r, p, keylen): ...
    def __init__(self, n, r, p): ...
    def run(self, secret, salt, keylen): ...
    def smix(self, input) -> Generator[None, None, Any]: ...
    def bmix(self, source, target) -> None: ...

__all__ = ["ScryptEngine"]



from typing import Any

class AttributeGetter:
    def __init__(self, attributes: dict[str, Any] | None = None) -> None: ...
    # This doesn't exist at runtime, but subclasses should define their own fields populated by attributes in __init__
    # Until that's done, keep __getattribute__ to fill in the gaps
    def __getattribute__(self, name: str, /) -> Any: ...



from _typeshed import FileDescriptorOrPath
from asyncio.events import AbstractEventLoop
from concurrent.futures import Executor
from os import PathLike
from typing import AnyStr

__all__ = [
    "abspath",
    "getatime",
    "getctime",
    "getmtime",
    "getsize",
    "exists",
    "isdir",
    "isfile",
    "islink",
    "ismount",
    "samefile",
    "sameopenfile",
]

async def exists(
    path: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...
) -> bool: ...
async def isfile(
    path: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...
) -> bool: ...
async def isdir(s: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...) -> bool: ...
async def islink(path: FileDescriptorOrPath) -> bool: ...
async def ismount(path: FileDescriptorOrPath) -> bool: ...
async def getsize(
    filename: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...
) -> int: ...
async def getmtime(
    filename: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...
) -> float: ...
async def getatime(
    filename: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...
) -> float: ...
async def getctime(
    filename: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...
) -> float: ...
async def samefile(
    f1: FileDescriptorOrPath, f2: FileDescriptorOrPath, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...
) -> bool: ...
async def sameopenfile(fp1: int, fp2: int, *, loop: AbstractEventLoop | None = ..., executor: Executor | None = ...) -> bool: ...
async def abspath(path: PathLike[AnyStr] | AnyStr) -> AnyStr: ...



from _typeshed import Incomplete
from typing import ClassVar

from .. import Command

class rotate(Command):
    description: str
    user_options: ClassVar[list[tuple[str, str, str]]]
    boolean_options: ClassVar[list[str]]
    match: Incomplete
    dist_dir: Incomplete
    keep: Incomplete
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
